
% ----------------------------------------------------------------------- %
%
%
%                   U N C E R T A I N   K I N G D O M :  
%            N O W C A S T I N G   G D P   &   R E V I S I O N S 
%
%             Silvia Miranda-Agrippino, Niki Anesti & Ana Galvao
%
%               
%                  r e p l i c a t i o n   c o d e
%
%           
%  RA-DFM
% ========
%
% Produces nowcasts for the first four GDP releases at each quarters: 
% - GDP releases and revisions are tracked in sequence. 
% - News contribution to nowcast updates at each release. 
% - Revisions for other variables are used to update the information set at 
%   each release, but are not explicitly modelled.
%
% 
%
%
%
% miranda (2016) silvia.miranda-agrippino@bankofengland.co.uk
% ----------------------------------------------------------------------- %



clear 
clc


addpath([pwd,'/SUBROUTINES'])
addpath([pwd,'/DATA'])



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %
%set basic coordinates
DFMinit.rawDataFile         ='UKdata_RTVintages_July2017.xlsx'; %if new data read
DFMinit.dataFile            ='UKdata_RTVintages_July2017';      %if in mat file (already transformed)


%plot variables that enter the model
DFMinit.plotVariables       =false;                


%set targets
DFMinit.country             ='UK';
DFMinit.targetSeries        ={'GDPP','GDPA','GDP6','GDP12'};


%set sample bounds
DFMinit.startEstimate       =datenum(1992,01,01); 
DFMinit.startForecast       =datenum(2006,09,01);
DFMinit.endForecast         =datenum(2017,02,01);




% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %
%build dataset
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %

dataStructure               =readRealTimeData_UKversion(DFMinit);
load(DFMinit.dataFile);


dataStructure.selectN       =(dataStructure.datadescr);



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %

%set file info
DFMinit.endMatFileName      ='DEMO'; %this is the name of the output mat file 
DFMinit.outputFolderName    ='DEMO';
DFMinit.readmeInfo          ='DEMO';


%set folder to store results
if ~exist(DFMinit.outputFolderName,'dir')
    
    mkdir(DFMinit.outputFolderName);
end


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %
%unpack dimensions
nM                  =dataStructure.nM;                          %number of monthly variables
nQ                  =dataStructure.nQ;                          %number of quarterly variables
nT                  =length(dataStructure.referencedates);      %number of reference periods (static)
nV                  =length(dataStructure.vintagedates);        %number of vintages


%find target series
isTarget            =find(ismember(dataStructure.seriesid,DFMinit.targetSeries))';
noT                 =numel(isTarget);                           %number of target series




% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %
% set DFM specification
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %

DFMinit.transform   =dataStructure.transformations; %for initial conditions [log diff yoy]
DFMinit.nM          =nM;    
DFMinit.nQ          =nQ;    
DFMinit.max_iter    =1000;                          %max EM iterations



%loadings restrictions (restrictions are written as R1*LAMBDA=R2)

DFMinit.R1          =[2 -1 0 0 0; 3 0 -1 0 0; 2 0 0 -1 0; 1 0 0 0 -1];
DFMinit.R2          =zeros(4,1);



%blocks in factor structure
DFMinit.blocks                              =double(dataStructure.blocks);
DFMinit.blocks(:,all(DFMinit.blocks==0))    =[];



%factors & lags
DFMinit.r           =[1 1 1];                       %number of factors per block
DFMinit.p           =1;                             %number of lags in factors VAR




%choose estimates over rolling or expanding time windows
DFMinit.estimWindow  ='expanding'; %string, either rolling or expanding
DFMinit.windowLength =10;          %in years



%detect GDP releases
DFMinit.isGDPrelease =dataStructure.isgdprelease;





% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %
% MODEL ESTIMATES
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - %

vintageID       =dataStructure.vintagedates;
dataVintages    =dataStructure.vintages;
dates           =dataStructure.referencedates;
lastVintage     =dataStructure.vintages{end,:};


%containers for results
DFMbackcast     =cell(noT,1); 
DFMnowcast      =cell(noT,1); 
DFMforecast     =cell(noT,1); 

ARbackcast      =cell(noT,1); 
ARnowcast       =cell(noT,1); 
ARforecast      =cell(noT,1); 


%target-specific
sizeO           =ceil(1.5*(nM+nQ/3)*(months( vintageID(1),vintageID(end) )+1 )); %~ output size
it              =1;

for j =1:noT
    
    DFMbackcast{j}  =nan(sizeO,16); 
    DFMnowcast{j}   =nan(sizeO,16); 
    DFMforecast{j}  =nan(sizeO,16); 

    ARbackcast{j}   =nan(sizeO,1); 
    ARnowcast{j}    =nan(sizeO,1); 
    ARforecast{j}   =nan(sizeO,1); 
end

%factor updates
realActivityFactor  =nan(nT,nV);



%%
% . IS . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .%

%in sample parameters estimates
DFM_Pnew = EM_DHDFM_SS_block_restrMQ_VARidio(lastVintage,DFMinit,[]);


   
% . OOS . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . %

for q =1:nV
    
       
    %prelims
    if q>1
        previousVdate   =vintageID(q-1); 
        currentVdate    =vintageID(q);
        
        oldDataVintage  =newDataVintage;        
    end   
    

    %print info
    if q==1
        
        fprintf(1,' - * -  - * -  - * -  - * -  - * -  - * -  - * -  - * - - * - \n');
        fprintf(1,'OOS started for %s: evaluation sample: %s to %s\n',DFMinit.country,...
            datestr(vintageID(1),'dd-mmm-yy'),datestr(vintageID(end),'dd-mmm-yy'));

    elseif month(currentVdate)~=month(previousVdate)

        fprintf(1,'now running: %s\n',datestr(currentVdate,'mmm-yy'));
    end
    
    
    newDataVintage      =dataVintages{q};
    releaseTime         =vintageID(q);
    
    %set estimation bounds (beginning of estimation sample)
    if q==1 || year(currentVdate)~=year(previousVdate)
        

        switch DFMinit.estimWindow

            case 'rolling'
                startE =datenum(year(vintageID(q))-DFMinit.windowLength+1,1,1);
                startE =find(dates==startE,1,'first');

                fprintf(1,'SS estimate from: %s\n',datestr(dates(startE),'mmm-yy'));


            case 'expanding'
                startE =find(dates==DFMinit.startEstimate,1,'first');

                fprintf(1,'SS estimate from: %s\n',datestr(dates(startE),'mmm-yy'));

        end
    end
    
    
    %cut from newly-defined estimation start
    relevantDates       =dates(startE:end);    
    newDataVintage      =newDataVintage(startE:end,:);
    
    
    %find current quarter
    currentQ            =datenum(year(releaseTime),round((month(releaseTime)+1)/3)*3,1);
    iQ                  =find(ismember(relevantDates,currentQ)); %NOWCAST
    
    DFMinit.Q0          =[year(currentQ) month(currentQ)]; 
    

   
    %update model parameters at every new year
    if q==1 || year(currentVdate)~=year(previousVdate)
 
         
        % * DFM * %
        DFM_Pnew        = EM_DHDFM_SS_block_restrMQ_VARidio(newDataVintage(1:iQ+3,:),DFMinit,[]);                 
        
        DFM_Pnew.dataName =dataStructure.datadescr;
        
 
        %real-time factor updates
        tempFactor      =DFM_Pnew.States(:,1);
         
        realActivityFactor(1:length(tempFactor),q)=tempFactor;
        % .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
         
         
        % AR %
        beta =cell(noT,1);
        for j=1:noT
             
            yGDP        =newDataVintage(isfinite(newDataVintage(:,isTarget(j))),isTarget(j));
         
            beta{j}     =[ones(length(yGDP)-1,1) yGDP(1:end-1,:)]\yGDP(2:end,:);
        end
    end     
    

    
    if q > 1
        
        if year(currentVdate)~=year(previousVdate) && strcmp(DFMinit.estimWindow,'rolling')
            
            oldDataVintage = oldDataVintage(13:end,:); %move forward start date at new year (oldDataVintage is defined over previous year)
        end
        
                
        %find new data releases between vintages
        nanPreRelease   =isnan(oldDataVintage); %nan pattern pre release
        nanPostRelease  =isnan(newDataVintage); %nan pattern post release
        
        [rT,rN] =find(nanPreRelease~=nanPostRelease); %position of releases

        for j=1:numel(rN)
            if rT(j)<find(~isnan(oldDataVintage(:,rN(j))),1,'last'); %is some other random nan but not a release
                
                rN(j)=nan;
            end
        end
        rT=rT(~isnan(rN)); rN=rN(~isnan(rN));

        
        
        % * contribution to forecasts updates (data news) * %

        CFU =News_DFM(oldDataVintage(1:iQ+3,:),newDataVintage(1:iQ+3,:),DFM_Pnew,iQ,isTarget);
        % .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
        
        
        % * forecast uncertainty (density) * %

        DF  =Density_DFM(newDataVintage(1:iQ+3,:),DFM_Pnew,iQ,isTarget);
        % .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

        
        %real-time factor updates
        tempFactor =CFU.ss.update.S_smooth(1,:);
        
        realActivityFactor(1:length(tempFactor),q)=tempFactor;

       
        %revise AR forecasts
        for j=1:noT
            
            if isnan(newDataVintage(iQ-3,isTarget(j))) %not yet released
                
                ARbackcast{j}(q,:) =[1 newDataVintage(iQ-6,isTarget(j))]*beta{j}; %backcast
                
            else
                
                ARbackcast{j}(q,:) =newDataVintage(iQ-3,isTarget(j));
            end
            
            ARnowcast{j}(q,:) =[1 ARbackcast{j}(q,:)]*beta{j};             %nowcast
            ARforecast{j}(q,:)=[1 ARnowcast{j}(q,:)]*beta{j};              %forecast  
        
        
            %
            releasesAtDate =numel(CFU.rN);
            
            tableItemF =nan(releasesAtDate,16);
            tableItemN =nan(releasesAtDate,16);
            tableItemB =nan(releasesAtDate,16);
        
            
            %rearrange and store RA-DFM & AR output
            for r=1:releasesAtDate
                
                %find release in input set
                [~,ir] =ismember(CFU.releaseID(r),DFM_Pnew.dataName);
                
                %
                releasedValue   =CFU.releasedValue(r);                
                modelNews       =CFU.releasedValue(r)-CFU.expectedValue(r);
                

                %
                tableItemF(r,:) =[m2xdate(vintageID(q)) releasedValue ir ...                             %date-time, value released and ID of series being released,
                                 modelNews CFU.forecast.update(:,j)' DF.forecast.bands(j,:)...           %data news, GDP forecast update, coverage bands
                                 CFU.forecast.weights(r,j) CFU.forecast.impacts(r,j)...                  %weights and impact of data news on GDP forecast update
                                 m2xdate(relevantDates(iQ+3,:)) lastVintage(startE+iQ+3-1,isTarget(j)) ARforecast{j}(q)]; %reference quarter, GDP outturn, AR forecast
                                 
                tableItemN(r,:) =[m2xdate(vintageID(q)) releasedValue ir ...                             %date-time, value released and ID of series being released,
                                 modelNews CFU.nowcast.update(:,j)' DF.nowcast.bands(j,:)...             %data news, GDP forecast update, coverage bands
                                 CFU.nowcast.weights(r,j) CFU.nowcast.impacts(r,j)...                    %weights and impact of data news on GDP forecast update
                                 m2xdate(relevantDates(iQ,:)) lastVintage(startE+iQ-1,isTarget(j)) ARnowcast{j}(q)];  %reference quarter, GDP outturn, AR forecast
       
                tableItemB(r,:) =[m2xdate(vintageID(q)) releasedValue ir ...                             %date-time, value released and ID of series being released,
                                 modelNews CFU.backcast.update(:,j)' DF.backcast.bands(j,:) ...          %data news, GDP forecast update, coverage bands
                                 CFU.backcast.weights(r,j) CFU.backcast.impacts(r,j)...                  %weights and impact of data news on GDP forecast update
                                 m2xdate(relevantDates(iQ-3,:)) lastVintage(startE+iQ-3-1,isTarget(j)) ARforecast{j}(q)]; %reference quarter, GDP outturn, AR forecast
            
            end
            
            
            % update containers
            DFMforecast{j}(it:it+releasesAtDate-1,:)  =tableItemF;
            DFMnowcast{j}(it:it+releasesAtDate-1,:)   =tableItemN;
            DFMbackcast{j}(it:it+releasesAtDate-1,:)  =tableItemB;
                        
        end
        
        it=it+releasesAtDate;

    end
end



%trim nan entries
for j=1:noT
    
    DFMforecast{j} =DFMforecast{j}(~isnan(DFMforecast{1}(:,1)),:);
    DFMnowcast{j}  =DFMnowcast{j}(~isnan(DFMnowcast{1}(:,1)),:);
    DFMbackcast{j} =DFMbackcast{j}(~isnan(DFMbackcast{1}(:,1)),:);
end





%drop unnecesary items
dataStructure =rmfield(dataStructure,'vintages');

%clean up workspace
clearvars -except DFMinit dataStructure DFMforecast DFMnowcast DFMbackcast realActivityFactor

%save forecast output
save([pwd '/' DFMinit.outputFolderName '/' DFMinit.endMatFileName])

% DFMnowcast{j}=[(1)    Release Date Time (xls format)
%                (2)    Released Value (x_i)
%                (3)    Release ID
%                (4)    Data News [x_i - E(x_i)]
%                (5:6)  GDP Forecast Update [y_{t|v} y_{t|v-1}]
%                (7:11) Coverage Bands [ Lower(95) Lower(90) 0 Upper(90) Upper(95)]
%                (12)   Weight of Data News
%                (13)   Impact of Data News on GDP Forecast Update
%                (14)   Reference Quarter (xls format)
%                (15)   GDP Outturn
%                (16)   AR(1) forecast


