
function res=Density_DFM(V_new,ssModelParams,nHorizon,targetSeries)
%
% computes density forecasts for quarterly target series. Allows for
% multiple quarterly targets. conditional on e_t and u_t being normally
% distributed, all distribution statistics are available in closed form

% INPUT: V_new          =data vintage post release
%        ssModelParams  =structure with parameters of state space
%        nHorizon       =forecast horizons for nowcast
%        targetSeries   =position of target series

% 
% y_{t}=C*s_{t}  +u_{t};  u_{t}~N(0,Q)
% s_{t}=A*s_{t-1}+e_{t};  e_{t}~N(0,R)

% P_{t|t-1}  = E[(s_{t}-s_{t|t-1})(s_{t}-s_{t|t-1})'] = AP_{t-1|t-1}A'+Q
% H_{t|t-1}  = E[(y_{t}-y_{t|t-1})(y_{t}-y_{t|t-1})'] = CP_{t-1|t-1}C'+R
% B_{t|t-1}' = E[(s_{t}-s_{t|t-1})(y_{t}-y_{t|t-1})'] = P_{t|t-1}C'


% OUTPUT (structure)
% rT rN releaseID                         %details of release
% releasedValue expectedValue innovation  %news components
%
% backcast.update    backcast.weights    backcast.impacts 
% nowcast.update     nowcast.weights     nowcast.impacts 
% forecast.update    forecast.weights    forecast.impacts 


% miranda (2016) silvia.miranda-agrippino@bankofengland.co.uk


%unpack state space
C  =ssModelParams.C; 
R  =ssModelParams.R';


%get forecast states' variance: P_{t|t-1}
SS_update =kfMultipleLags(V_new,ssModelParams,0);


%states covariance (contemporaneous)
P =SS_update.P_smooth(2:end);     %contains initial values


%current quarter forecast
t =nHorizon;

%target series
j =targetSeries;


%forecast variance % H_{t|t-1}=E[(y_{t}-y_{t|t-1})(y_{t}-y_{t|t-1})']
H(:,1) =diag(C(j,:) *P{t-3} *C(j,:)'+ R(j,j));
H(:,2) =diag(C(j,:) *P{t}   *C(j,:)'+ R(j,j));     %nowcast
H(:,3) =diag(C(j,:) *P{t+3} *C(j,:)'+ R(j,j));


%point forecasts
Y =SS_update.X_smooth([t-3 t t+3],j)';


%forecast bands 
cLevels =[-1.65 -.99 0 .99 1.65];     nC=numel(cLevels);

res.forecast.bands = kron(ones(1,nC),Y(:,3)) + kron(cLevels, sqrt(H(:,3)));
res.nowcast.bands  = kron(ones(1,nC),Y(:,2)) + kron(cLevels, sqrt(H(:,2)));
res.backcast.bands = kron(ones(1,nC),Y(:,1)) + kron(cLevels, sqrt(H(:,1)));


%distribution moments
res.forecast.point    =Y(:,3);
res.forecast.variance =H(:,3);

res.nowcast.point     =Y(:,2);
res.nowcast.variance  =H(:,2);

res.backcast.point    =Y(:,1);
res.backcast.variance =H(:,1);

