function StateSpaceP_new=EM_DHDFM_SS_block_restrMQ_VARidio(X,modelSpec,StateSpaceP_old)

% EM algorithm for mixed frequency (M/Q) dynamic factor model
% idiosyncratic for the subsequent GDP releases follow a VAR(1)
%
% DFM:
% X_{t} =\Lambda F_{t} + v_{t};  
% F_{t} =A1 F_{t-1} + ... + Ap F_{t-p} + u_{t};
% v_{t} =D1 v_{t-1} + e_{t};
% F_{t} =[f_{1t}',...,f_{rt}']';

% u_{t}~iid(0,\Omega); e~iid(0,\Psi); (\Psi diagonal)
% X=[Nx1]; \Lambda_{j}=[Nxr]; F=[rx1]; e=[Nx1]; u=[rx1]; Ai=[rxr]; E(Fe')=0
% active restriction on quarterly loadings (nR=#of factor lags involved)
%
%
% SSF:
% X_{t} =C S_{t} + e_{t}
% S_{t} =A S_{t-1} + u_{t}
% e_{t}~N(0,R); u_{t}~N(0,Q); P_{t}=var(S_{t});
%
%
% vector of states:
% S_{t} =[F_{t},F_{t-1},...,F_{t-max(p-1,nR)},...       factors and their lags [r x max(p,nR)+1]
%         vM_{t},vM_{t-1},...                           monthly idio [nM x 1]
%         vQ_{t},vQ_{t-1},...,vQ_{t-max(s-1,nR)}]';     quarterly idio & lags [nQ x nR+1]
%
% nS   =nSf+nSiM+nSiQ;                                  %number of states
% nSf  =sum(r.*(max(p,nR)+1)); 
% nSiM =nM*s; 
% nSiQ =nQ*(nR+1); 
%
% X=[N x 1]; C=[N x nS]; S=[nS x 1]; e=[N x 1]; R=[N x N]; 
% A=[nS x nS]; u=[nS x 1]; Q=[nS x nS]; 

% INPUT:
% X         =[TxN] matrix of data
% modelSpec =structure
%
% r =number of factors
% p =number of lags in factor VAR

% OUTPUT: structure containing parameters estimates (SSF)
% S       =smoothed states
% P       =smoothed states variance
% Xhat    =smoothed data
% C,R,A,Q =system matrices


% built on Banbura & Modugno (2010)

% miranda 2019 silvia.miranda-agrippino@northwestern.edu




%--------------------------------------------------------------------------
% initialize

T    =size(X,1); 
mX   =nanmean(X); 
vX   =nanstd(X);

xNaN =bsxfun(@minus,X,mX);          %original data, contains NaNs
xNaN =bsxfun(@rdivide,xNaN,vX);


if ~isempty(StateSpaceP_old)
    
    SystemMatrices_init.C   =StateSpaceP_old.C;
    SystemMatrices_init.R   =StateSpaceP_old.R; 
    SystemMatrices_init.A   =StateSpaceP_old.A; 
    SystemMatrices_init.Q   =StateSpaceP_old.Q; 
    SystemMatrices_init.S   =StateSpaceP_old.S; 
    SystemMatrices_init.P   =StateSpaceP_old.P;
    
else
    
    SystemMatrices_init     =initializeEM(xNaN,modelSpec);
end



%--------------------------------------------------------------------------
%EM algorithm


% set parameters
nmaxit          =modelSpec.max_iter; 
dLLthreshold    =1E-3;  %for likelihood convergence

it=0; 

%loglikelihood
llconverged     =0; 
loglklhd_old    =-inf; 
loglklhd_store  =nan(nmaxit,1);


%remove leading and ending nans for the estimation (only removes rows which
%have all missing values)
optNaN.method   =3;                
optNaN.k        =3;

y_est=remNaNs_spline(xNaN,optNaN);


%EM
while it<=nmaxit && llconverged==0

    
    [SystemMatrices_end,loglklhd]=EMalgorithm(y_est,modelSpec,SystemMatrices_init);
    
    
    %update matrices
    SystemMatrices_init.S =SystemMatrices_end.S;
    SystemMatrices_init.P =SystemMatrices_end.P;
   
    SystemMatrices_init.C =SystemMatrices_end.C;
    SystemMatrices_init.R =SystemMatrices_end.R;
    SystemMatrices_init.A =SystemMatrices_end.A;
    SystemMatrices_init.Q =SystemMatrices_end.Q;
    
    
    
    %check likelihood convergence
    llconverged =EMconverged(loglklhd,loglklhd_old,dLLthreshold,1,it);

    loglklhd_old        =loglklhd; 
    loglklhd_store(it+1)=loglklhd;
    
    it=it+1;
    
    if it==nmaxit && ~llconverged

        fprintf(1,'EM DID NOT CONVERGE: maximum number of iterations reached!\n');
        fprintf(1,'Iterations allowed: %i; Tolerance Level %1.2e \n',nmaxit,dLLthreshold);
    end
    
end

if llconverged

    fprintf(1,'EM converged at iteration %i of %i\n',it,nmaxit);
    fprintf(1,'Tolerance Level %1.2e \n',dLLthreshold);
end


% last run of kalman filter & smoother 
% @ iteration j states are estimated using system matrices @ iteration (j-1))
KalmanFilterOutput      =kalman_filter(xNaN,SystemMatrices_end);
KalmanSmootherOutput    =kalman_smoother(KalmanFilterOutput);


%load results
StateSpaceP_new.S       =KalmanSmootherOutput.S_smooth(:,1); %S_{0|0}
StateSpaceP_new.P       =SystemMatrices_end.P;               %P_{0|0}

StateSpaceP_new.C       =SystemMatrices_end.C;
StateSpaceP_new.R       =SystemMatrices_end.R;
StateSpaceP_new.A       =SystemMatrices_end.A;
StateSpaceP_new.Q       =SystemMatrices_end.Q;


StateSpaceP_new.States  =KalmanSmootherOutput.S_smooth(:,2:end)'; %states
StateSpaceP_new.Xhat    =((KalmanSmootherOutput.S_smooth(:,2:end)'*...
                         SystemMatrices_end.C').*kron(vX,ones(T,1)))+kron(mX,ones(T,1)); %estimated X

StateSpaceP_new.mX      =mX; 
StateSpaceP_new.vX      =vX; 






% -----*-----*-----*-----*-----*-----*-----*-----*-----*-----*-----*----- %
%                              child fcts                                 %
% -----*-----*-----*-----*-----*-----*-----*-----*-----*-----*-----*----- %






function [SystemMatrices_end,loglklhd]=EMalgorithm(x,modelSpec,SystemMatrices_init)

%returns smoothed states and their variance + estimates of the system's matrices



[T,n]       =size(x);      


%unpack model specification
r       =modelSpec.r;               %number of factors
p       =modelSpec.p;               %number of lags in factor VAR


blocks  =modelSpec.blocks;          %block restrictions



%restrictions are written as R1*C = R2
R1      =modelSpec.R1;              %matrix for loading restrictions(1) 
R2      =modelSpec.R2;              %matrix for loading restrictions(2)
Rpoly   =[1 R1(:,1)'];


q       =size(R1,1);                %number of lags in M-Q map


nM      =modelSpec.nM;              %number of monthly variables
nQ      =modelSpec.nQ;              %number of quarterly variables
nB      =size(blocks,2);            %number of blocks






% *     *     *     *   E X P E C T A T I O N   *     *     *     *

KalmanFilterOutput          =kalman_filter(x,SystemMatrices_init);
KalmanSmootherOutput        =kalman_smoother(KalmanFilterOutput);


%smoothed states and their variance 
S_smooth                    =KalmanSmootherOutput.S_smooth;     %E[S(t)]
P_smooth                    =KalmanSmootherOutput.P_smooth;     %Var[S(t)]
PP_smooth                   =KalmanSmootherOutput.PP_smooth;    %Cov[S(t),S(t-1)]


%log likelihood
loglklhd                    =KalmanFilterOutput.loglklhd;




% *     *     *     *   M A X I M I Z A T I O N    *     *     *     *
%[Shumway&Stoffer(2000)]


nSf     =sum(r.*(max(p,q)+1));      %number of states (factors & own lags)
nSm     =nM;                        %number of states (idio M)
nSq     =nQ*(q+1);                  %number of states (idio Q & own lags)

nS      =nSf+nSm+nSq;               %total number of states



%coefficients matrices
C_end   =zeros(n,nS); 
A_end   =zeros(nS,nS); 
Q_end   =zeros(nS,nS);
P_end   =zeros(nS,nS);


P       =zeros(nS,nS); 
Pl      =zeros(nS,nS); 
PPl     =zeros(nS,nS);

for t=1:T

    P   =P+P_smooth{t+1};           %sum(Var[S(t)])
    Pl  =Pl+P_smooth{t};            %sum(Var[S(t-1)])
    PPl =PPl+PP_smooth{t+1};        %sum(Cov[S(t),S(t-1)])
end




% * ~ ~ * ~ ~ * ~ ~ *  S T A T E   E Q U A T I O N  * ~ ~ * ~ ~ * ~ ~ *
    
% * ~ ~ * ~ ~ * ~ ~ *     F  A  C  T  O  R  S       * ~ ~ * ~ ~ * ~ ~ *



nSfi     =cumsum([1 r.*(max(p,q)+1)]); %location of factors in C A Q

for b=1:nB
    
    
    rb          =r(b);              %number of factors per block
    k           =max(p,q);          %number of effective lags
    
       
    %position of factors in states' vector
    F_starts    =nSfi(b);
    F_ends      =nSfi(b)+rb*p-1;
    
 
    
    %SUFFICIENT STATISTICS
       
    
    %E[F(t)F(t)']
    E_FF    =S_smooth( F_starts:F_ends,2:end )*S_smooth( F_starts:F_ends,2:end )'+...
             P( F_starts:F_ends, F_starts:F_ends );               
    
    %E[F(t-1)F(t-1)']
    E_FlFl  =S_smooth( F_starts:F_ends,1:end-1 )*S_smooth( F_starts:F_ends,1:end-1 )'+...
             Pl( F_starts:F_ends,F_starts:F_ends );              
    
    %E[F(t)F(t-1)']
    E_FFl   =S_smooth( F_starts:F_ends,2:end )*S_smooth( F_starts:F_ends,1:end-1 )'+...
             PPl( F_starts:F_ends,F_starts:F_ends );             
        
    
         
    %relevant coefficients
    tempA   =E_FFl/E_FlFl; 
    tempQ   =(1/T)*(E_FF-tempA*E_FFl');

    

    %update A matrix for relevant block
    blockA                      =zeros(rb*(k+1),rb*(k+1));
    blockA(1:rb,1:rb*p)         =tempA(1:rb,1:rb*p); 
    blockA(rb+1:end,1:end-rb)   =eye(rb*k);
    
    
    %update A (factors)
    A_end(nSfi(b):nSfi(b+1)-1,nSfi(b):nSfi(b+1)-1) = blockA;
    
    
    
    
    %update Q matrix for relevant block
    blockQ                      =zeros(rb*(k+1),rb*(k+1));
    blockQ(1:rb,1:rb)           =tempQ(1:rb,1:rb);
    
    
    %update Q (factors)
    Q_end(nSfi(b):nSfi(b+1)-1,nSfi(b):nSfi(b+1)-1) = blockQ;
    
    
    
    
    %update P matrix for relevant block
    blockP                      =( eye((rb*(k+1))^2) - kron(blockA,blockA) )\blockQ(:);
    
    P_end(nSfi(b):nSfi(b+1)-1,nSfi(b):nSfi(b+1)-1) = reshape(blockP,rb*(k+1),rb*(k+1));

end







% * ~ ~ * ~ ~ * ~ ~ *  S T A T E   E Q U A T I O N  * ~ ~ * ~ ~ * ~ ~ *

% * ~ ~ * ~ ~ * ~ ~ *   I D I O S Y N C R A T I C   * ~ ~ * ~ ~ * ~ ~ *

%idioM
for i=1:nM
    
    
    %position of idioM in states' vector
    iM_starts   =nSf+i;
    iM_ends     =nSf+i;
    

    
    %SUFFICIENT STATISTICS

    
    %E[eM(t)eM(t)']
    E_eMeM   =S_smooth( iM_starts:iM_ends,2:end )*S_smooth( iM_starts:iM_ends,2:end )'+...
                P( iM_starts:iM_ends,iM_starts:iM_ends ); 
          
    %E[eM(t-1)eM(t-1)']
    E_eMleMl =S_smooth( iM_starts:iM_ends,1:end-1 )*S_smooth( iM_starts:iM_ends,1:end-1 )'+...
                Pl( iM_starts:iM_ends,iM_starts:iM_ends ); 
          
    %E[eM(t)eM(t-1)']
    E_eMeMl  =S_smooth( iM_starts:iM_ends,2:end )*S_smooth( iM_starts:iM_ends,1:end-1 )'+...
                PPl( iM_starts:iM_ends,iM_starts:iM_ends ); 
          
          
          
          
    %relevant coefficients
    tempA   =E_eMeMl/E_eMleMl;  %AR(1)
    tempQ   =(1/T)*(E_eMeM-tempA*E_eMeMl');
    
    
    %update A (idio M)
    A_end( iM_starts:iM_ends,iM_starts:iM_ends ) =tempA;

    
    %update Q (idio M)
    Q_end( iM_starts:iM_ends,iM_starts:iM_ends ) =tempQ;


    %update P (idio M)
    P_end( iM_starts:iM_ends,iM_starts:iM_ends ) =(1-tempA^2)\tempQ;
end



%idioQ
for i=nM+1:n
    
    %quarterly other than GDP have idio ~ AR(1)
    if ~ismember(i,find(modelSpec.isGDPrelease))
           
        
        
        %position of idio Q in states' vector
        iQ_starts   =nSf+nSm+1+(i-nM-1)*(q+1);
        iQ_ends     =nSf+nSm+1+(i-nM-1)*(q+1);
        


        % SUFFICIENT STATISTICS

        
        %E[eQ(t)eQ(t)']
        E_eQeQ   =S_smooth( iQ_starts:iQ_ends,2:end)*S_smooth( iQ_starts:iQ_ends,2:end)'+...
                    P( iQ_starts:iQ_ends,iQ_starts:iQ_ends ); 

        %E[eQ(t-1)eQ(t-1)']
        E_eQleQl =S_smooth( iQ_starts:iQ_ends,1:end-1)*S_smooth( iQ_starts:iQ_ends,1:end-1)'+...
                    Pl( iQ_starts:iQ_ends,iQ_starts:iQ_ends ); 

        %E[eQ(t)eQ(t-1)']
        E_eQeQl  =S_smooth( iQ_starts:iQ_ends,2:end)*S_smooth( iQ_starts:iQ_ends,1:end-1)'+...
                    PPl( iQ_starts:iQ_ends,iQ_starts:iQ_ends ); 

              
              
              
        %relevant coefficients
        tempA   =E_eQeQl/E_eQleQl;  %AR(1)
        tempQ   =(1/T)*(E_eQeQ-tempA*E_eQeQl');
    

        %update position of idio Q in states' vector    
        iQ_ends     =nSf+nSm+(i-nM)*(q+1);


        %update A matrix for relevant block
        blockA                  =zeros(q+1,q+1); 
        blockA(1,1)             =tempA; 
        blockA(2:end,1:end-1)   =eye(q);

        %update A (idio Q)
        A_end( iQ_starts:iQ_ends,iQ_starts:iQ_ends ) =blockA;

          
        %update Q matrix for relevant block
        blockQ                  =zeros(q+1,q+1); 
        blockQ(1,1)             =tempQ; 

        %update Q (idio Q)
        Q_end( iQ_starts:iQ_ends,iQ_starts:iQ_ends ) =blockQ;  

                

        %initialize P (idio Q)
        blockP                   =(eye((q+1)^2) - kron(blockA,blockA))\blockQ(:);
        
        P_end(iQ_starts:iQ_ends,iQ_starts:iQ_ends) =reshape(blockP,q+1,q+1);

    end
end



%idio for GDP releases ~ VAR(1)
nG          =sum(modelSpec.isGDPrelease);       %number of GDP releases in VAR
k           =q+1;                               %number of parameters in each equation


%position of idio G in states' vector
iG_starts   =nSf+nSm+(nQ-nG)*k+1;
iG_ends     =nSf+nSm+nSq;



% SUFFICIENT STATISTICS

%E[eQ(t)eQ(t)']
E_eQeQ   =S_smooth( iG_starts:k:iG_ends,2:end)*S_smooth( iG_starts:k:iG_ends,2:end)'+...
            P( iG_starts:k:iG_ends,iG_starts:k:iG_ends ); 

%E[eQ(t-1)eQ(t-1)']
E_eQleQl =S_smooth( iG_starts:k:iG_ends,1:end-1)*S_smooth( iG_starts:k:iG_ends,1:end-1)'+...
            Pl( iG_starts:k:iG_ends,iG_starts:k:iG_ends ); 

%E[eQ(t)eQ(t-1)']
E_eQeQl  =S_smooth( iG_starts:k:iG_ends,2:end)*S_smooth( iG_starts:k:iG_ends,1:end-1)'+...
            PPl( iG_starts:k:iG_ends,iG_starts:k:iG_ends ); 


        
%relevant coefficients        
tempA   =E_eQeQl/E_eQleQl;  %VAR(1)
tempQ   =(1/T)*(E_eQeQ-tempA*E_eQeQl');


    
%update A matrix for relevant block (rearrange positions)
blockA  =zeros(nG*k,nG*k); 

for j=1:nG

    blockA( k*(j-1)+1, 1:k:(iG_ends-iG_starts+1) )  =tempA(j,:);
    blockA( k*(j-1)+2:k*j, k*(j-1)+1:k*j-1 )        =eye(q);

end



%update Q matrix for relevant block
blockQ                              =zeros(nG*k,nG*k); 
blockQ(1:k:nG*k,1:k:nG*k)           =tempQ;



%update P matrix for relevant block
blockP = (eye((nG*k)^2) - kron(blockA,blockA))\blockQ(:);
blockP = reshape(blockP,nG*k,nG*k);



%update A
A_end( iG_starts:iG_ends,iG_starts:iG_ends ) =blockA;

  

%update Q
Q_end( iG_starts:iG_ends,iG_starts:iG_ends ) =blockQ;
          

%update P
P_end( iG_starts:iG_ends,iG_starts:iG_ends ) =blockP;




  
  
      
  
% * ~ ~ * ~ ~ *  M E A S U R E M E N T   E Q U A T I O N  * ~ ~ * ~ ~ *

% * ~ ~ * ~ ~ *             F  A  C  T  O  R  S           * ~ ~ * ~ ~ *

% * ~ ~ * ~ ~ *                      &                    * ~ ~ * ~ ~ *

% * ~ ~ * ~ ~ *         I D I O S Y N C R A T I C         * ~ ~ * ~ ~ *

 
 
% define superblocks as combination of factors, and group variables
% according to which one they load on distinguishing bwn M and Q


k                   =max(p,q);          %number of effective lags


superB              =unique(blocks,'rows'); 
nSB                 =size(superB,1);


%factors and their lags
MloadPerSB          =zeros(nSB,nSf);    
QloadPerSB          =zeros(nSB,nSf); 
QloadPerSBr         =zeros(nSB,nSf);


%position of factors in states vector
nSfi                =cumsum([1 r.*(max(p,q)+1)]); 
nSfiB               =repmat(r,nSB,1).*superB;

for b=1:nB

    %position of monthly loadings in super block
    MloadPerSB(:,nSfi(b):nSfi(b)+r(b)-1)           =repmat(superB(:,b),1,r(b));    
    
    %position of quarterly loadings in super block
    QloadPerSB(:,nSfi(b):nSfi(b)+(k+1)*r(b)-1)     =repmat(superB(:,b),1,(k+1)*r(b));

    %position of restricted quarterly loadings in super block
    QloadPerSBr(:,nSfi(b):nSfi(b)+(q+1)*r(b)-1)    =repmat(superB(:,b),1,(q+1)*r(b));
end



%idiosyncratic
iMloadPerSB         =zeros(n,nS); 
iQloadPerSB         =zeros(n,nS); 

iMloadPerSB(1:nM,nSf+1:nSf+nSm)   =eye(nM); 
iQloadPerSB(nM+1:n,nSf+nSm+1:end) =kron(eye(nQ),ones(1,q+1));


%build identifiers -- these will pick the right columns of the states
%vector depending on which variables load on which states in each block
MloadPerSB          =logical(MloadPerSB); 
QloadPerSB          =logical(QloadPerSB);       %quarterly loading for each super block
QloadPerSBr         =logical(QloadPerSBr);      %quarterly restricted loading for each super block
iMloadPerSB         =logical(iMloadPerSB);    
iQloadPerSB         =logical(iQloadPerSB);      %idiosyncratic quarterly loading for each super block

for sb=1:nSB
    
    
    %find all variables that load on all factors in super block
    selectV     =find(ismember(blocks,superB(sb,:),'rows'));
    
    MinSB       =selectV(selectV<=nM);          %monthly variables in block
    nMSB        =numel(MinSB); 
    
    QinSB       =selectV(selectV>nM);           %quarterly variables in block
    nQSB        =numel(QinSB);
    

    %vec(C)=[sum(kron(E(FF'),W))]^{-1}[vec(sum(Wy*E(F')))-vec(sum(W*E(F*e')))]
    %vec(C)=[MC1]^{-1}[MC2]
    
    
    
    % M O N T H L Y   L O A D I N G S
    
    if~isempty(MinSB)
        
        MC1     =zeros(nMSB*sum(MloadPerSB(sb,:)),nMSB*sum(MloadPerSB(sb,:))); 
        MC2     =zeros(nMSB,sum(MloadPerSB(sb,:)));
        
        for t=1:T
            
            %track missing values
            Wx =x(t,MinSB)';    W =diag(~isnan(Wx));    Wx(isnan(Wx)) =0;
            
            
            %
            MC1 =MC1 + kron(S_smooth(MloadPerSB(sb,:),t+1)*S_smooth(MloadPerSB(sb,:),t+1)'+...
                             P_smooth{t+1}(MloadPerSB(sb,:),MloadPerSB(sb,:)),W);
            %
            MC2 =MC2 + Wx*S_smooth(MloadPerSB(sb,:),t+1)'-...
                       W*(S_smooth(any(iMloadPerSB(MinSB,:),1),t+1)*S_smooth(MloadPerSB(sb,:),t+1)'+...
                           P_smooth{t+1}(any(iMloadPerSB(MinSB,:),1),MloadPerSB(sb,:)));
            
        end
        
        %update C matrix for monthly variables
        vecC    =MC1\MC2(:);         
        tempC   =reshape(vecC,nMSB,sum(MloadPerSB(sb,:)));
        
        
        %update C
        C_end(MinSB,MloadPerSB(sb,:))=tempC;
        
    end
    
    
    
    % Q U A R T E R L Y   L O A D I N G S
    
    if~isempty(QinSB)
        
        
        %same formulas as above
        
        MC1     =zeros(nQSB*sum(QloadPerSB(sb,:)),nQSB*sum(QloadPerSB(sb,:))); 
        MC1r    =zeros(nQSB*sum(QloadPerSBr(sb,:)),nQSB*sum(QloadPerSBr(sb,:))); 
        MC2     =zeros(nQSB,sum(QloadPerSB(sb,:)));
        
        for t=1:T
            
            %track missing values
            Wx =x(t,QinSB)';     W =diag(~isnan(Wx));     Wx(isnan(Wx)) =0;
            
            %
            MC1  =MC1+kron(S_smooth(QloadPerSB(sb,:),t+1)*S_smooth(QloadPerSB(sb,:),t+1)'+...
                  P_smooth{t+1}(QloadPerSB(sb,:),QloadPerSB(sb,:)),W);
            %
            MC1r =MC1r+kron(S_smooth(QloadPerSBr(sb,:),t+1)*S_smooth(QloadPerSBr(sb,:),t+1)'+...
                  P_smooth{t+1}(QloadPerSBr(sb,:),QloadPerSBr(sb,:)),W);
            %
            MC2  =MC2+Wx*S_smooth(QloadPerSB(sb,:),t+1)'-...
                  W*(kron(eye(nQSB),Rpoly)*S_smooth(any(iQloadPerSB(QinSB,:),1),t+1)*S_smooth(QloadPerSB(sb,:),t+1)'+...
                  kron(eye(nQSB),Rpoly)*P_smooth{t+1}(any(iQloadPerSB(QinSB,:),1),QloadPerSB(sb,:)));
        end
        
        
        %update C matrix for quarterly variables
        vecC    =MC1\MC2(:); 
        tempC   =reshape(vecC,nQSB,sum(QloadPerSB(sb,:)));
        
        
        %impose restrictions (loading identifiers)
        nSfiBi  =nSfiB(sb,:); 
        nSfiBi  =nSfiBi(nSfiBi~=0); 
        
        %number of restricted coefficients
        nCr     =(q+1)*sum(nSfiBi); 
        
        
        %position of restricted coefficients
        irBr    =[1 cumsum(nSfiBi)*q+1]; 
        irBc    =[1 cumsum(nSfiBi)*(q+1)+1];
        
        
        R1B     =zeros(nCr-sum(nSfiBi),nCr); 
        R2B     =zeros(nCr-sum(nSfiBi),1);
        
        for ir=1:length(nSfiBi)
            
            R1B(irBr(ir):irBr(ir+1)-1,irBc(ir):irBc(ir+1)-1) =kron(R1,eye(nSfiBi(ir))); 
            R2B(irBr(ir):irBr(ir+1)-1)                       =kron(R2,ones(nSfiBi(ir),1));
        end
        
        R1B     =kron(eye(nQSB),R1B); 
        R2B     =kron(ones(nQSB,1),R2B); 
        
        
        %all C coefficients
        tempCall                        =zeros(nSf,nQSB); 
        tempCall(QloadPerSB(sb,:),:)    =tempC'; 
        
        
        %C coefficients subject to restriction
        tempCr                          =zeros(nSf,nQSB); 
        tempCr(QloadPerSBr(sb,:),:)     =tempCall(QloadPerSBr(sb,:),:); 

        tempCr                          =tempCr(:); 
        tempCr(tempCr==0)               =[];
        
        
        %impose loadings restrictions C 
        tempCr                          =tempCr - MC1r\R1B'/(R1B/MC1r*R1B')*(R1B*tempCr-R2B);
        tempCr                          =reshape(tempCr,nCr,nQSB); 
        
        
        %replace
        tempCall(QloadPerSBr(sb,:),:)   =tempCr;
        
        
        %update C matrix for quarterly variables
        C_end(QinSB,QloadPerSB(sb,:))   =tempCall(QloadPerSB(sb,:),:)';    

    end
end


%update C idiosyncratic
C_end(1:nM,nSf+1:nSf+nSm)           =eye(nM);
C_end(nM+1:nM+nQ,nSf+nSm+1:nS)      =kron(eye(nQ),Rpoly);


%update R (small number)
R_end                               =diag(1e-04*ones(n,1));




%load structure with results
SystemMatrices_end.S =S_smooth(:,1);
SystemMatrices_end.P =P_end;
SystemMatrices_end.C =C_end; 
SystemMatrices_end.R =R_end;
SystemMatrices_end.A =A_end; 
SystemMatrices_end.Q =Q_end;

% -----*-----*-----*-----*-----*-----*-----*-----*-----*-----*-----*----- %








function [converged,decrease] = EMconverged(loglik,previous_loglik,threshold,check_increased,iteration)
%
% We have converged if the slope of the log-likelihood function falls below 'threshold',
% i.e., |f(t) - f(t-1)| / avg < threshold,
% where avg = (|f(t)| + |f(t-1)|)/2 and f(t) is log lik at iteration t.
% 'threshold' defaults to 1e-4.
%
% This stopping criterion is from Numerical Recipes in C p423
%
% If we are doing MAP estimation (using priors), the likelihood can decrase,
% even though the mode of the posterior is increasing.

if nargin<3, threshold=1e-4;     end
if nargin<4, check_increased=1;  end
if nargin<5, iteration=1;        end

converged =false; 
decrease  =false;

if check_increased
    if loglik-previous_loglik < -1e-3 % allow for a little imprecision
        
        fprintf(1,'(!) likelihood decreased from %6.4f to %6.4f at iteration %i (!)\n',previous_loglik,loglik,iteration);
        decrease=true;
        
    end
end

delta_loglik =abs(loglik-previous_loglik);
avg_loglik   =(abs(loglik)+abs(previous_loglik)+eps)/2;


if (delta_loglik/avg_loglik)<threshold; converged =true; end
% -----*-----*-----*-----*-----*-----*-----*-----*-----*-----*-----*----- %








function SystemMatrices_init = initializeEM(x,modelSpec)

%returns initial values for system matrices C R A Q and states' variance P


%NOTE: this is assuming that the variables in x are ordered such that the
%monthly ones come first, then all the quarterly which are not GDP releases,
%then the consecutive GPD releases


n       =size(x,2);      


%unpack model specification
r       =modelSpec.r;               %number of factors
p       =modelSpec.p;               %number of lags in factor VAR


blocks  =modelSpec.blocks;          %blocks restrictions


%restrictions are written as R1*C = R2
R1      =modelSpec.R1;              %matrix for loading restrictions(1) 
R2      =modelSpec.R2;              %matrix for loading restrictions(2)
Rpoly   =[1 R1(:,1)'];


q       =size(R1,1);                %number of lags in M-Q map


nM      =modelSpec.nM;              %number of monthly variables
nQ      =modelSpec.nQ;              %number of quarterly variables
nB      =size(blocks,2);            %number of blocks







%initialize state space
nSf     =sum(r.*(max(p,q)+1));      %number of states (factors & own lags)
nSm     =nM;                        %number of states (idio M)
nSq     =nQ*(q+1);                  %number of states (idio Q & own lags)

nS      =nSf+nSm+nSq;               %total number of states


%coefficients matrices
C_init  =zeros(n,nS);           
A_init  =zeros(nS,nS); 
Q_init  =zeros(nS,nS); 
P_init  =zeros(nS,nS);




%remove nans for initialization with principal components
%replace with MA(k) after deleting rows with leading or ending nans
optNaN.method   =2;                 
optNaN.k        =3;

[xSpline,iNaN]  =remNaNs_spline(x,optNaN); 


xNaN            =xSpline;
xNaN(iNaN)      =nan;               %contains original nan pattern 

T               =size(xNaN,1);      %number of available data points



% initialize state space w\ principal components & least squares

tempX           =xSpline;           %data without nans
tempXnan        =xNaN;              %data with original nans pattern

tempFactors     =nan(T,nSf);

%NOTE: the only procedure that uses the spline data is the principal
%components extraction. C R A Q only use actual data points


nSfi     =cumsum([1 r.*(max(p,q)+1)]); %location of factors in C A Q
for b=1:nB
    
  
    
    % * ~ ~ * ~ ~ *  M E A S U R E M E N T   E Q U A T I O N  * ~ ~ * ~ ~ *

    % * ~ ~ * ~ ~ *             F  A  C  T  O  R  S           * ~ ~ * ~ ~ *
    
    
    
    
    rb      =r(b);                  %number of factors per block
    ib      =nSfi(b);               %location of factors in C A Q per block
    k       =max(p,q);              %number of effective factors lags
    

    tempBlk =find(blocks(:,b)); 
    MinB    =tempBlk(tempBlk<=nM);  %monthly variables in block b
    QinB    =tempBlk(tempBlk>nM);   %quarterly variables in block b
    
    

    %factors are initialized on the monthly variables
    if isempty(MinB)
    
        error('There are no monthly variables in the chosen block')
    end
        
    vX      =cov(tempX(:,MinB)); 
    [V,~]   =eigs(vX,rb); 
    F       =tempX(:,MinB)*V; 
    F       =F(:,1:rb);             %principal components for factors

    
    
    %factors in measurement eqn
    lagF_Meqn =nan(T-q,rb*(q+1)); %[F_{t},F_{t-1},...,F_{t-h}]';
    for j=0:q

        lagF_Meqn(:,rb*j+1:rb*(j+1)) =F((q+1)-j:end-j,:);
    end
    
    
    for im =1:length(MinB)
    
        xM      =tempXnan(q+1:end,MinB(im)); 
        fM      =lagF_Meqn(~isnan(xM),1:rb); 
        
        xM      =xM(~isnan(xM),:);
        
        
        %monthly loadings
        tempC   =fM\xM; 

        %initialize C
        C_init( MinB(im), ib:ib+rb-1 ) = tempC';    
    end
    
    
    %quarterly loadings (with restrictions)
    R1B   =kron(R1,eye(rb)); 
    R2B   =kron(R2,ones(rb,1));
    
    for iq =1:length(QinB)
        
        
        xQ      =tempXnan(q+1:end,QinB(iq)); 
        fQ      =lagF_Meqn(~isnan(xQ),:); 
        
        xQ      =xQ(~isnan(xQ),:);
        

        %quarterly loadings (unrestricted)
        tempC   =fQ\xQ;
        
        %apply restrictions (restricted least squares)
        tempC   =tempC - ( fQ(:,1:rb*(q+1))'*fQ(:,1:rb*(q+1)) )\R1B'/...
                         ( R1B / (fQ(:,1:rb*(q+1))' * fQ(:,1:rb*(q+1))) * R1B' )*( R1B*tempC - R2B );


        %initialize C
        C_init( QinB(iq), ib:ib+rb*(q+1)-1 ) = tempC';
        
    end
    
    
    
    %orthogonalize for next block extraction
    tempF          =[zeros( q,rb*(q+1) ); lagF_Meqn];
    
    
    %`data' orthogonal to factors in preceding block
    tempX          =tempX - tempF*C_init(:,ib:ib+rb*(q+1)-1)';
    
    
    %`data' orthogonal to factors in preceding block with original nan pattern
    tempXnan       =tempX; 
    tempXnan(iNaN) =NaN;
    
    
    %store
    tempFactors(:,ib:ib+rb*(q+1)-1) =tempF;
    
    
    
    % * ~ ~ * ~ ~ * ~ ~ *  S T A T E   E Q U A T I O N  * ~ ~ * ~ ~ * ~ ~ *
    
    % * ~ ~ * ~ ~ * ~ ~ *     F  A  C  T  O  R  S       * ~ ~ * ~ ~ * ~ ~ *
    
    
    
    
    %factors VAR
    lagF_Seqn =nan(T-p,rb*p); %[F_{t},F_{t-1},...,F_{t-h}]';
    for j=1:p

        lagF_Seqn(:,rb*(j-1)+1:rb*j) =F((p+1)-j:end-j,:);
    end    
    
    tempA       =[ones(T-p,1) lagF_Seqn]\F(p+1:end,:);
    
    u           =F(p+1:end,:)-[ones(T-p,1) lagF_Seqn]*tempA; 
    tempQ       =cov(u);
    
    
    
    %initialize A for each block
    blockA                      =zeros(rb*(k+1),rb*(k+1));
    blockA(1:rb,1:rb*p)         =tempA(2:end,:)'; 
    blockA(rb+1:end,1:end-rb)   =eye(rb*k);
    
    %initialize A (factors)
    A_init( ib:ib+rb*(k+1)-1 , ib:ib+rb*(k+1)-1 ) = blockA;
    
    
    
    
    %initialize Q for each block
    blockQ                      =zeros(rb*(k+1),rb*(k+1));
    blockQ(1:rb,1:rb)           =tempQ;
    
    %initialize Q (factors)
    Q_init( ib:ib+rb*(k+1)-1 , ib:ib+rb*(k+1)-1 ) = blockQ;
    

    
    %initialize P (factors)
    blockP                      =( eye((rb*(k+1))^2) - kron(blockA,blockA) )\blockQ(:);
    
    P_init( ib:ib+rb*(k+1)-1 , ib:ib+rb*(k+1)-1 ) = reshape(blockP,rb*(k+1),rb*(k+1));

end






% * ~ ~ * ~ ~ *  M E A S U R E M E N T   E Q U A T I O N  * ~ ~ * ~ ~ *

% * ~ ~ * ~ ~ *         I D I O S Y N C R A T I C         * ~ ~ * ~ ~ *


%initialize C (idiosyncratic)
C_init(1:nM,nSf+1:nSf+nSm)      =eye(nM);
C_init(nM+1:nM+nQ,nSf+nSm+1:nS) =kron(eye(nQ),Rpoly);


%initialize R (small)
R_init                          =diag(1e-04.*ones(n,1)); 







% * ~ ~ * ~ ~ * ~ ~ *  S T A T E   E Q U A T I O N  * ~ ~ * ~ ~ * ~ ~ *

% * ~ ~ * ~ ~ * ~ ~ *   I D I O S Y N C R A T I C   * ~ ~ * ~ ~ * ~ ~ *


E =tempX-tempFactors*C_init(:,1:nSf)';  %orthogonal to all the factors
             
%NOTE: this only works if variables enter the X matrix in the following
%order: first all the monthly, then all the quarterly which are not GDP,
%then all the GDP releases

%idioM
for i=1:nM
    
    idioM       =E(:,i);
    idioM       =idioM(~isnan(idioM));
    
    tempA       =[ones(numel(idioM)-1,1) idioM(1:end-1)]\idioM(2:end);   %AR(1)
    
    uM          =idioM(2:end)-[ones(numel(idioM)-1,1) idioM(1:end-1)]*tempA; 
    tempQ       =cov(uM);
    
    
    %initialize A (idio M)
    A_init(nSf+i,nSf+i) =tempA(2);

    
    %initialize Q (idio M)
    Q_init(nSf+i,nSf+i) =tempQ;
        

    %initialize P (idio M)
    P_init(nSf+i,nSf+i) =(1-tempA(2)^2)\tempQ;
end




%idioQ
for i=nM+1:n
    
    %quarterly other than GDP have idio ~ AR(1)
    if ~ismember(i,find(modelSpec.isGDPrelease)) 
        
        %find relevant idio Q
        idioQ       =E(:,i);
        idioQ       =idioQ(~isnan(idioQ));

        
        %position of idio Q in states' vector
        iQ_starts   =nSf+nSm+1+(i-nM-1)*(q+1);
        iQ_ends     =nSf+nSm+(i-nM)*(q+1);

        
        
        %relevant coefficients
        tempA       =[ones(numel(idioQ)-1,1) idioQ(1:end-1)]\idioQ(2:end); %AR(1)

        uQ          =idioQ(2:end)-[ones(numel(idioQ)-1,1) idioQ(1:end-1)]*tempA; 
        tempQ       =cov(uQ);

        

        %initialize A (idio Q)
        blockA                  =zeros((q+1),(q+1)); 
        blockA(1,1)             =tempA(2)'; 
        blockA(2:end,1:end-1)   =eye(q);

        A_init(iQ_starts:iQ_ends,iQ_starts:iQ_ends) =blockA;



        %initialize Q (idio Q)
        blockQ                  =zeros((q+1),(q+1)); 
        blockQ(1,1)             =tempQ; 

        Q_init(iQ_starts:iQ_ends,iQ_starts:iQ_ends) =blockQ;
        

        %initialize P (idio Q)
        tempP                   =(eye((q+1)^2) - kron(blockA,blockA))\blockQ(:);
        
        P_init(iQ_starts:iQ_ends,iQ_starts:iQ_ends) =reshape(tempP,q+1,q+1);

    end
end


%idio for GDP releases ~ VAR(1)
idioG       =E(:,modelSpec.isGDPrelease); 
idioG       =idioG(all(~isnan(idioG')),:);


nG          =size(idioG,2);                     %number of GDP releases in VAR


%position of idio G in states' vector
iG_starts   =nSf+nSm+(nQ-nG)*(q+1)+1;
iG_ends     =nSf+nSm+nSq;


tempA       =[ones(size(idioG,1)-1,1) idioG(1:end-1,:)]\idioG(2:end,:); %equations in columns

% tempA(2:end,:)= diag(.2*ones(nG,1)); %equations in columns

uG          =idioG(2:end,:)-[ones(size(idioG,1)-1,1) idioG(1:end-1,:)]*tempA; 
tempQ       =cov(uG);

tempA       =tempA(2:end,:)';    %remove intercept -- equations in rows

tempP       =(eye(nG^2)-kron(tempA,tempA))\tempQ(:);
tempP       =reshape(tempP,nG,nG);



%NOTE:  the computation of tempA, tempQ and tempP for idio G assumes that
%       for the GDP releases the vector of states is aligned as 
%       [e_t,e_{t-1},...,e_{t-q}] while in fact it is 
%       [e1_t,e1_{t-1},...,e1_{t-q},e2_t,e2_{t-1},...,e2_{t-q},...]
%       the lines below reshuffle the entries of all three matrices to 
%       match the actual composition of the states vector


%initialize A (idio G)
blockA      =zeros(nG*(q+1),nG*(q+1)); 
for j=1:nG
    
    blockA( (q+1)*(j-1)+1, 1:q+1:(iG_ends-iG_starts+1) )       =tempA(j,:); %equation in rows
    blockA( (q+1)*(j-1)+2:(q+1)*j, (q+1)*(j-1)+1:(q+1)*j-1 )   =eye((q+1)-1);
    
end

A_init(iG_starts:iG_ends,iG_starts:iG_ends) =blockA;




%initialize Q (idio G)
blockQ                                      =zeros(nG*(q+1),nG*(q+1)); 
blockQ(1:(q+1):nG*(q+1),1:(q+1):nG*(q+1))   =tempQ;


%initialize P (idio G)
blockP                                      =zeros(nG*(q+1),nG*(q+1)); 
blockP(1:(q+1):nG*(q+1),1:(q+1):nG*(q+1))   =tempP;


Q_init(iG_starts:iG_ends,iG_starts:iG_ends) =blockQ;
P_init(iG_starts:iG_ends,iG_starts:iG_ends) =blockP;
        





%initialize states (all)
S_init =zeros(nS,1);



%load structure
SystemMatrices_init.S =S_init; 
SystemMatrices_init.P =P_init;

SystemMatrices_init.C =C_init; 
SystemMatrices_init.R =R_init;
SystemMatrices_init.A =A_init; 
SystemMatrices_init.Q =Q_init;


