
function res=News_DFM(V_old,V_new,ssModelParams,nHorizon,targetSeries)
%
% computes projection and projection coefficients of (quarterly) target 
% series onto news for forecast, nowcast and backcast. Allows for
% multiple quarterly targets. Not optimized for monthly targets.
% built on Ba?bura and Modugno(2012) 

% INPUT: V_old          =vintage pre release
%        V_new          =vintage post release
%        ssModelParams  =structure with parameters of state space
%        nHorizon       =forecast horizons for nowcast
%        targetSeries   =position of target series

% 
% y_{t}=C*s_{t}  +u_{t};  u_{t}~N(0,Q)
% s_{t}=A*s_{t-1}+e_{t};  e_{t}~N(0,R)

% P_{t|t-1}  = E[(s_{t}-s_{t|t-1})(s_{t}-s_{t|t-1})'] = AP_{t-1|t-1}A'+Q
% H_{t|t-1}  = E[(y_{t}-y_{t|t-1})(y_{t}-y_{t|t-1})'] = CP_{t-1|t-1}C'+R
% B_{t|t-1}' = E[(s_{t}-s_{t|t-1})(y_{t}-y_{t|t-1})'] = P_{t|t-1}C'


% OUTPUT (structure)
% rT rN releaseID                         %details of release
% releasedValue expectedValue innovation  %news components
%
% backcast.update    backcast.weights    backcast.impacts 
% nowcast.update     nowcast.weights     nowcast.impacts 
% forecast.update    forecast.weights    forecast.impacts 

% ss.update %state-space update (fixed coefficients, new factor and
% forecasts)

% miranda (2014) smirandaagrippino@london.edu


%unpack state space
nS =size(ssModelParams.C,2); %number of states
n  =size(V_new,2);

Mx =ssModelParams.mX; 
Wx =ssModelParams.vX; 
C  =ssModelParams.C; 
R  =ssModelParams.R';


%find release
nanPreRelease  =isnan(V_old); %nan pattern pre release
nanPostRelease =isnan(V_new); %nan pattern post release
[rT,rN]        =find(nanPreRelease~=nanPostRelease); %position of release

for j=1:numel(rN)
    if rT(j)<find(~isnan(V_old(:,rN(j))),1,'last'); %is some other random nan but not a release

        rN(j)=nan;
    end
end
rT=rT(~isnan(rN)); rN=rN(~isnan(rN));


if isempty(rN)
    
    res.tR=[];
    res.rN=[]; 
    
    res.releaseID    =[];
    res.releasedValue=[]; 
    res.expectedValue=[];
    
    %
    res.backcast.update =[];
    res.backcast.weights=[];
    res.backcast.impacts=[];
    
    %
    res.nowcast.update =[];
    res.nowcast.weights=[];
    res.nowcast.impacts=[];
    
    %
    res.forecast.update =[];
    res.forecast.weights=[];
    res.forecast.impacts=[];
    
    %
    res.ss.update =[];
    
    return
end


releaseID =ssModelParams.dataName(rN); %released series identifier

lag =nHorizon-rT; %distance bwn release and nowcast horizon for covariance of states
k   =max([abs(lag); max(lag)-min(lag)]);


SS_old =kfMultipleLags(V_old,ssModelParams,k+3);
SS_new =kfMultipleLags(V_new,ssModelParams,0);

P_smoothNl  =SS_old.P_smoothNl;          %smoothed states covariance P_{t,t-k|T}
modelStates =SS_new.S_smooth(:,2:end)';  %factor updates



%covariance between data and states: E[(s_{t}-s_{t|t-l})(y_{t}-y_{t-l})']=P_{t-1|t-l}C'
B =nan(nS,numel(rN),3);
for i=1:size(lag,1)
    
    
    %nowcast
    h =abs(nHorizon-rT(i));   %which lag in covariance matrix    
    t =max([rT(i) nHorizon]); %which time for covariance matrix

    if  rT(i)>nHorizon;   B(:,i,2) =P_smoothNl{h+1}(:,:,t)*C(rN(i),1:nS)'; 
    else                  B(:,i,2) =P_smoothNl{h+1}(:,:,t)'*C(rN(i),1:nS)';   end

    
    %forecast
    h =abs((nHorizon+3)-rT(i));
    t =max([rT(i) nHorizon+3]);
    
    if  rT(i)>nHorizon+3; B(:,i,3) =P_smoothNl{h+1}(:,:,t)*C(rN(i),1:nS)';
    else                  B(:,i,3) =P_smoothNl{h+1}(:,:,t)'*C(rN(i),1:nS)';   end
    
    
    %backcast
    h =abs((nHorizon-3)-rT(i)); 
    t =max([rT(i) nHorizon-3]);
    
    if  rT(i)>nHorizon-3; B(:,i,1)=P_smoothNl{h+1}(:,:,t)*C(rN(i),1:nS)'; 
    else                  B(:,i,1)=P_smoothNl{h+1}(:,:,t)'*C(rN(i),1:nS)';    end
    
end


    
%covariance between data releases: E[(y_{t}-y_{t-l})(y_{t}-y_{t-l})']=CP_{t-1|t-l}C'+R
H  =nan(numel(rN),numel(rN),3); 
WW =nan(size(R));

for i=1:size(lag,1)
    for j=1:size(lag,1)
        
        t =max([rT(i),rT(j)]); %which time for covariance matrix
        if rN(i)==rN(j)&&rT(i)~=rT(j); WW(rN(i),rN(j))=0; else WW(rN(i),rN(j))=R(rN(i),rN(j)); end
        
        
        %nowcast
        tlag =nHorizon-rT; 
        h    =abs(tlag(i)-tlag(j)); %which lag in covariance matrix
        
        if rT(j)>rT(i); pp =P_smoothNl{h+1}(:,:,t); 
        else            pp =P_smoothNl{h+1}(:,:,t)';    end
        
        H(i,j,2) =C(rN(i),1:nS)*pp*C(rN(j),1:nS)'+WW(rN(i),rN(j));
        
        
        %forecast
        tlag =nHorizon+3-rT; 
        h    =abs(tlag(i)-tlag(j));
        
        if rT(j)>rT(i); pp =P_smoothNl{h+1}(:,:,t); 
        else            pp =P_smoothNl{h+1}(:,:,t)';    end
        
        H(i,j,3) =C(rN(i),1:nS)*pp*C(rN(j),1:nS)'+WW(rN(i),rN(j));        
        
        
        %backcast
        tlag =nHorizon-3-rT; 
        h    =abs(tlag(i)-tlag(j));
        
        if rT(j)>rT(i); pp =P_smoothNl{h+1}(:,:,t); 
        else            pp =P_smoothNl{h+1}(:,:,t)';    end
        
        H(i,j,1) =C(rN(i),1:nS)*pp*C(rN(j),1:nS)'+WW(rN(i),rN(j)); 
        
    end
end


%news I_{v+1}=y_{i,t}-E[y_{i,t}|\Omega_{v}]
releasedValue =zeros(n,1); 
expectedValue =zeros(n,1);

for i=1:size(rN,1)
    
    releasedValue(rN(i),1) =V_new(rT(i),rN(i));
    expectedValue(rN(i),1) =SS_old.X_smooth(rT(i),rN(i));
end


%innovations (standardized news) I_{v+1}/variance
innovations=nan(n,1);

for i=1:size(rN,1)
    
    normX_release =( V_new(rT(i),rN(i)) - Mx(rN(i)) )./ Wx(rN(i));
    normX_smooth  =( SS_old.X_smooth(rT(i),rN(i)) - Mx(rN(i)) )./ Wx(rN(i));
    
    innovations(rN(i))=normX_release-normX_smooth;
end


%weights E[y_{t}I_{v+1}]E[I_{v+1}I_{v+1}']^{-1} (multiplied by target variance)
%impacts E[y_{t}I_{v+1}]E[I_{v+1}I_{v+1}']^{-1}*I_{v+1}

%different targets on 3rd dimension
weights       =zeros(n,3,length(targetSeries)); 
scaledWeights =zeros(n,3,length(targetSeries)); 
impacts       =zeros(n,3,length(targetSeries));

for iS=1:length(targetSeries)
    for d=1:3
        
        weights(rN,d,iS)       =Wx(targetSeries(iS))*C(targetSeries(iS),1:nS)*B(:,:,d)/H(:,:,d); %kalman gain

        impacts(rN,d,iS)       =weights(rN,d,iS).*innovations(rN);
        
        scaledWeights(rN,d,iS) =weights(rN,d,iS)./Wx(rN)'; %scaled weights
        %alternatively impacts=scaledWeights*news
        
    end
end



%load output structure
res.rT =rT;
res.rN =rN; 

res.releaseID     =releaseID;
res.releasedValue =releasedValue(rN); 
res.expectedValue =expectedValue(rN);

res.innovations   =innovations(rN); 
res.modelStates   =modelStates;

%
res.ss.update     =SS_new;


%
res.nowcast.update  =[SS_old.X_smooth(nHorizon,targetSeries); SS_new.X_smooth(nHorizon,targetSeries)];
res.nowcast.weights =reshape(squeeze(scaledWeights(rN,2,:)),numel(rN),numel(targetSeries));
res.nowcast.impacts =reshape(squeeze(impacts(rN,2,:)),numel(rN),numel(targetSeries));
res.nowcast.gains   =reshape(squeeze(weights(rN,2,:)),numel(rN),numel(targetSeries));

%
res.forecast.update  =[SS_old.X_smooth(nHorizon+3,targetSeries); SS_new.X_smooth(nHorizon+3,targetSeries)];
res.forecast.weights =reshape(squeeze(scaledWeights(rN,3,:)),numel(rN),numel(targetSeries));
res.forecast.impacts =reshape(squeeze(impacts(rN,3,:)),numel(rN),numel(targetSeries));
res.forecast.gains   =reshape(squeeze(weights(rN,3,:)),numel(rN),numel(targetSeries));

%
for iS=1:length(targetSeries)
    if isnan(V_new(nHorizon-3,targetSeries(iS))) %not yet released
        
        res.backcast.update(:,iS) =[SS_old.X_smooth(nHorizon-3,targetSeries(iS));...
                                     SS_new.X_smooth(nHorizon-3,targetSeries(iS))];
    else
        res.backcast.update(:,iS) =[SS_old.X_smooth(nHorizon-3,targetSeries(iS));...
                                     V_new(nHorizon-3,targetSeries(iS))];        
    end
    
    res.backcast.weights(:,iS) =squeeze(scaledWeights(rN,1,iS));
    res.backcast.impacts(:,iS) =squeeze(impacts(rN,1,iS));
    res.backcast.gains(:,iS)   =squeeze(weights(rN,1,iS));
end
