
function dataStructure =readRealTimeData_UKversion(baseInfo)

% reads from xl file which contains a combination of revision triangles and
% static information on last available vintages and calendar of data releases.
% returns an ordered set of real time vintages and their timestamp.
%
% details from each build are saved into a log file created in the pwd
% which contains specific information on each of the series included.
%
% if charts are allowed a new folder is created within the pwd to collect
% them. charts give details on the real time vintages only.
%
%
% inputs:
% =======
%       readData: true/false
%    rawDataFile: name of xl spreadsheet collecting data (if readData true)
%       dataFile: name of mat file collecting data (if readData false)
%        country: string
%   targetSeries: string CAN IT BE A CELL?
%  plotVariables: true/false
%  startEstimate: datenum
%  startForecast: datenum
%    endForecast: datenum
% 
%
% original: miranda (2015) smirandaagrippino@london.edu
% updated : miranda (2017) silvia.miranda-agrippino@bankofengland.co.uk


rawDataFile =baseInfo.rawDataFile; %input data file (xl)
dataFile    =baseInfo.dataFile;    %output data file (mat)

if baseInfo.plotVariables
    
    %create subfolder to host charts
    baseInfo.chartsFolderName =[pwd '/DATA/plots'];
    
    if ~exist(baseInfo.chartsFolderName,'dir')
        
        mkdir(baseInfo.chartsFolderName);
    end
end


%load summary
[tempData,tempTxt]     =xlsread(rawDataFile,'summary');

baseInfo.dataList      =tempTxt(2:end,1);           %full list available
baseInfo.dataDescr     =tempTxt(2:end,2); 
baseInfo.dataFrequency =tempTxt(2:end,4);
baseInfo.isRealTime    =logical(tempData(:,1)); 
baseInfo.isGDPrelease  =logical(tempData(:,2)); 
baseInfo.dataSelect    =logical(tempData(:,3));      %used in model
baseInfo.dataTransform =logical(tempData(:,4:6)); 
baseInfo.dataBlocks    =logical(tempData(:,7:end));


%load releases calendar
[tempData,~] =xlsread(rawDataFile,'releasesCalendar');

releaseCal.refP         =x2mdate(tempData(:,1));%reference period
releaseCal.releaseDates =x2mdate(tempData(:,2:end));






%load last vintage for all variables
[tempData,~]  =xlsread(rawDataFile,'staticVintage');

lastVintage.data  =tempData(:,2:end); 
lastVintage.dates =x2mdate(tempData(:,1));



%create log file where construction details are saved
logFileName =['UKRT_Build.log'];
logFileName =strrep(logFileName,'-',''); 
logFileName =strrep(logFileName,' ',''); 
logFileName =strrep(logFileName,':','');

fid=fopen(logFileName,'w+'); baseInfo.logFileID=fid;

fprintf(fid,' +  -  -  -  +  -  -  -  +  -  -  -  +  -  -  -  +  -  -  -  + \r\n \r\n');
fprintf(fid,'              REAL TIME DATA SET FOR %s\r\n\r\n',baseInfo.country);
fprintf(fid,' +  -  -  -  +  -  -  -  +  -  -  -  +  -  -  -  +  -  -  -  + \r\n \r\n');

fprintf(fid,['Data read starts in %s \r\n' ...
             'Out of Sample span from %s to %s \r\n' ...
             'Real time vintages available for %i out of %i series. \r\n\r\n'],...
             datestr(baseInfo.startEstimate),datestr(baseInfo.startForecast),...
             datestr(baseInfo.endForecast),sum(baseInfo.isRealTime(baseInfo.dataSelect)),sum(baseInfo.dataSelect));

fprintf(fid,'build started at %s\r\n',datestr(rem(now,1)));
fprintf(fid,' . . . . . . . . . . .  \r\n\r\n\r\n');



% .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  %
%            load raw data and build set of revision triangles
% .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  %

%prepare holders
dataRevisionTriangles =cell(sum(baseInfo.dataSelect),1); %data vintages
vintageDates          =cell(sum(baseInfo.dataSelect),1); %release dates
vintageRefP           =cell(sum(baseInfo.dataSelect),1); %reference periods

for j=1:length(baseInfo.dataList)
    
    if baseInfo.dataSelect(j)
        
%         tic
%         
        idCode =baseInfo.dataList{j}; 
%         fprintf(1,' .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . \r\n');
%         fprintf(1,'building vintages for %s: series %i of %i\r\n\r\n',...
%                     baseInfo.dataDescr{j},sum(baseInfo.dataSelect(1:j)),sum(baseInfo.dataSelect));
%         
        %add build info to log file
        fprintf(fid,' .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . \r\n');
        fprintf(fid,'building vintages for %s: series %i of %i\r\n\r\n',...
                    baseInfo.dataDescr{j},sum(baseInfo.dataSelect(1:j)),sum(baseInfo.dataSelect));


        if baseInfo.isRealTime(j)
            
            tempRes =buildRealTimeVintage(baseInfo,idCode,lastVintage,releaseCal);
        else
            
            fprintf(fid,'(!) no real time info available: building pseudo vintages (!)\r\n\r\n');
            tempRes =buildPseudoRTimeVintage(baseInfo,idCode,lastVintage,releaseCal);
        end
        
        
        if tempRes.dataSelect(j)
                        
            %add vintages to holders
            dataRevisionTriangles{j} =tempRes.dataVintages; 
            vintageDates{j}          =tempRes.releaseDates; 
            vintageRefP{j}           =tempRes.referencePeriods;
        else
            
            baseInfo.dataSelect(j)=tempRes.dataSelect(j);
        end
        
%         toc
    end
end

fprintf(fid,' .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  . \r\n');
fprintf(fid,'\r\n build ended at %s\r\n',datestr(rem(now,1)));

fprintf(fid,' ---     ---     ---     --- --- --- ---     ---     ---     --- \r\n');
fprintf(fid,' - -     - -     - -     - -   END   - -     - -     - -     - - \r\n');
fprintf(fid,' ---     ---     ---     --- --- --- ---     ---     ---     --- \r\n');
fclose(fid);




%select relevant series
dataRevisionTriangles =dataRevisionTriangles(baseInfo.dataSelect);
vintageDates          =vintageDates(baseInfo.dataSelect);


% .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  %
%     build final data vintages to be used in estimation and forecast
% .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  %

%vintages in OOS span
vintageID =nan(sum(baseInfo.dataSelect),ceil((baseInfo.endForecast-baseInfo.startForecast)/12));

for j=1:sum(baseInfo.dataSelect)
    
  tempV=vintageDates{j}; 
   
  if ~isempty(tempV)
      
      vintageID(j,1:length(tempV))=tempV;
  end
end
vintageID =unique(vintageID(~isnan(vintageID))); %vintages in evaluation (date of each vintage)


%populate with variables

fullDataVintages =cell(length(vintageID),1); 

tempT =size(dataRevisionTriangles{1},1); 

for t=1:length(vintageID)
    
    iV    =vintageID(t);
    tempV =nan(tempT,sum(baseInfo.dataSelect));
    
    for j=1:sum(baseInfo.dataSelect)
        
        idx =find(vintageDates{j}<=iV,1,'last');
        if ~isempty(idx)
            
            tempV(:,j)     =dataRevisionTriangles{j}(:,idx);
            %             tempV(:,j)     =dataRevisionTriangles{j}(1:tempT,idx);
        else
            
            idx2           =find(~isnan(dataRevisionTriangles{j}(:,1)),1,'last');
            %            idx2           =find(~isnan(dataRevisionTriangles{j}(1:tempT,1)),1,'last');
            
            tempDataSeries =dataRevisionTriangles{j}(:,1); tempDataSeries(idx2)=nan;
            %             tempDataSeries =dataRevisionTriangles{j}(1:tempT,1); tempDataSeries(idx2)=nan;
            tempV(:,j)     =tempDataSeries;
        end
    end
    
    fullDataVintages{t} =tempV;
end

%load final data structure
dataStructure.vintages        =fullDataVintages;
dataStructure.vintagedates    =vintageID;
dataStructure.referencedates  =vintageRefP{1};
dataStructure.nM              =sum(strcmp(baseInfo.dataFrequency,'M') & baseInfo.dataSelect);
dataStructure.nQ              =sum(strcmp(baseInfo.dataFrequency,'Q') & baseInfo.dataSelect);
dataStructure.transformations =baseInfo.dataTransform(baseInfo.dataSelect,:);
dataStructure.blocks          =baseInfo.dataBlocks(baseInfo.dataSelect,:);
dataStructure.seriesid        =baseInfo.dataList(baseInfo.dataSelect);
dataStructure.datadescr       =baseInfo.dataDescr(baseInfo.dataSelect);
dataStructure.isgdprelease    =baseInfo.isGDPrelease(baseInfo.dataSelect);

%save dataset
save([pwd '/DATA/' dataFile],'baseInfo','dataRevisionTriangles','vintageDates','dataStructure')







%-child functions---------------------------------------------------------%

function res=buildRealTimeVintage(baseInfo,idCode,lastVintage,releaseCal)

%unpack general inputs
rawDataFile =baseInfo.rawDataFile; 
fid         =baseInfo.logFileID;
startE      =baseInfo.startEstimate;
startF      =baseInfo.startForecast; 
endF        =baseInfo.endForecast;


%unpack series-specific information
iS             =ismember(baseInfo.dataList,idCode);
iTransform     =baseInfo.dataTransform(iS,:); 
refP           =releaseCal.refP; 
iCal           =releaseCal.releaseDates(:,iS);
res.dataSelect =baseInfo.dataSelect; 

%last vintage
lastV     =lastVintage.data(:,iS); 
lastVrefP =lastVintage.dates;

%trim unnecessary observations (last vintage)
if startE>lastVrefP(1)
    
    cutData   =lastVrefP>=startE; 
    lastV     =lastV(cutData); 
    lastVrefP =lastVrefP(cutData);
end

%read from xl spreadsheet
[tempData,~]=xlsread(rawDataFile,idCode);

realTimeSetDates =x2mdate(tempData(1,2:end)); %release dates


if strcmp(baseInfo.dataFrequency(iS),'Q')
    
    %WARNING: 
    %I'm assuming quarterly data are recorded on the third month of each quarter 
    
    realTimeOriginalSet    =kron(tempData(2:end,2:end),[nan;nan;1]); 
    tempP                  =datevec(x2mdate(tempData(2:end,1)));
    realTimeSetRefP(:,1:3) =kron(tempP(:,1:3),ones(3,1));
    realTimeSetRefP(:,2)   =realTimeSetRefP(:,2)+repmat([-2;-1;0],length(tempP),1);
    realTimeSetRefP        =datenum(realTimeSetRefP);
else
    
    realTimeOriginalSet    =tempData(2:end,2:end); 
    realTimeSetRefP        =x2mdate(tempData(2:end,1));
end

%trim unnecessary observations (real-time vintages)
if startE>realTimeSetRefP(1)
    
    cutData             =realTimeSetRefP>=startE; 
    realTimeOriginalSet =realTimeOriginalSet(cutData,:); 
    realTimeSetRefP     =realTimeSetRefP(cutData);
    cutData             =find(all(isnan(realTimeOriginalSet)),1,'last');
    
    if ~isempty(cutData)
        
        realTimeOriginalSet =realTimeOriginalSet(:,cutData+1:end); 
        realTimeSetDates    =realTimeSetDates(cutData+1:end);
    end
end

%plot variables
if baseInfo.plotVariables
    
    Tstart         =max(lastVrefP(1),realTimeSetRefP(1)); 
    Tend           =min(lastVrefP(end-3),realTimeSetRefP(end-3));
    commonTimeline =lastVrefP(lastVrefP>=Tstart & lastVrefP<=Tend);

    allVplot  =realTimeOriginalSet(realTimeSetRefP>=Tstart & realTimeSetRefP<=Tend,:);
    
    if strcmp(baseInfo.dataFrequency(iS),'Q')
        
        allVplot  =kron(allVplot(3:3:end,:),ones(3,1));
    end
    
    figure;
    plot(commonTimeline,allVplot)
    set(gca,'XTick',commonTimeline(1:60:end),'FontSize',8,'Layer','top'); datetick('x','yyyy','keepticks');
    grid on; xlim([commonTimeline(1) commonTimeline(end)]);
    title([idCode,' :: real-time vintages'],'FontSize',8,'FontWeight','normal')

    set(gcf,'PaperPosition',[-1.5 -1 21 15]) %leftMargin topMargin figureLength figureHeight
    set(gcf,'PaperSize', [18 13]); %xSize ySize
    print(gcf,'-dpdf',[baseInfo.chartsFolderName,'\',idCode,'.pdf']); 
    
    close(gcf)

end


%time bounds
NoP        =max(length(lastVrefP),ceil(1.1*(endF-startE)/365*12));              %number of reference periods
firstVdate =datenum(year(realTimeSetDates(1)),month(realTimeSetDates(1)),1);    %date of first vintage
endDatesV  =realTimeSetDates(realTimeSetDates>=startF & realTimeSetDates<=endF);%release dates in OOS span
NoPV       =0;                                                                  %number of pseudo vintages

if firstVdate>startF %use also dates from static release calendar to fill up OOS span

    pseudoDates =iCal(refP<realTimeSetRefP(find(~isnan(realTimeOriginalSet(:,1)),1,'last')))';
    
    if ~all(isnan(pseudoDates))
        endDatesV   =[pseudoDates endDatesV]; 
        NoPV        =length(pseudoDates);
    end
end
NoV =length(endDatesV); %number of vintages


%build final data vintage set combining true and pseudo releases
endDataVraw =nan(NoP,NoV); 

if strcmp(baseInfo.dataFrequency(iS),'M')
    
    firstDataPointInV     =realTimeSetRefP(find(~isnan(realTimeOriginalSet(:,1)),1,'first'));
    firstDataPointInLastV =lastVrefP(find(~isnan(lastV),1,'first'));
    
elseif strcmp(baseInfo.dataFrequency(iS),'Q')
    
    firstDataPointInV     =realTimeSetRefP(find(~isnan(realTimeOriginalSet(:,1)),1,'first')-2);
    firstDataPointInLastV =lastVrefP(find(~isnan(lastV),1,'first')-2);
end



%case0:  data series begins after the beginning of the evaluation sample
%======

if min(firstDataPointInLastV,firstDataPointInV)>startF
    
    %record action in build log
    fprintf(fid,'(!) not enough data points for %s: removing series from set (!)\r\n',idCode);
    
    temp =find(lastVrefP==min(firstDataPointInLastV,firstDataPointInV));
    fprintf(fid,'(!) push start of OOS at least to %s to include it (!)\r\n',datestr(lastVrefP(temp+6)));
     
    %remove from set
    res.dataSelect(iS) =false; 
    return
end



%case1:  data series in real time vintages start at least at the beginning 
%======  of the evaluation sample

if firstDataPointInV<=startE

    if strcmp(baseInfo.dataFrequency(iS),'M')
    
    endDataVraw(1:sum(realTimeSetRefP>=startE),ismember(endDatesV,realTimeSetDates))=...
        realTimeOriginalSet(realTimeSetRefP>=startE,... %all reference points above startE
        ismember(realTimeSetDates,endDatesV));          %all available vintages between startF and endF 
    
    elseif strcmp(baseInfo.dataFrequency(iS),'Q')
    
        endDataVraw(1:sum(realTimeSetRefP>=startE),ismember(endDatesV,realTimeSetDates))=...
        realTimeOriginalSet(realTimeSetRefP>=startE,... %all reference points above startE
        ismember(realTimeSetDates,endDatesV));          %all available vintages between startF and endF 
    
    end
end   


  


%case2:  data series in real time vintages start later than the beginning of
%======  the last vintage --> substitute the initial missing values with fitted  
%        ones obtained using the last vintage as a reference point

case2=false;

if firstDataPointInLastV<firstDataPointInV  && strcmp(baseInfo.dataFrequency(iS),'M')

%    record action in build log
    fprintf(fid,'(!) data in real time vintages start in %s \r\n',datestr(firstDataPointInV));
    fprintf(fid,'(!) filling up from %s using projection on last vintage \r\n\r\n',datestr(firstDataPointInLastV));
 
    Tstart =max(firstDataPointInLastV,firstDataPointInV); 
    Tend   =min(lastVrefP(end),realTimeSetRefP(end));
    tempLV =lastV(lastVrefP>=Tstart & lastVrefP<=Tend); 
    iLV    =~isnan(tempLV);
    
    tempRtV =realTimeSetDates(realTimeSetDates>=startF & realTimeSetDates<=endF); 
    startV  =find(ismember(realTimeSetDates,tempRtV(1)));
    
    for t=length(tempRtV):-1:1
        
        tempV =realTimeOriginalSet(realTimeSetRefP>=Tstart & realTimeSetRefP<=Tend,t+startV-1); 
        iV    =~isnan(tempV);
        
        if t==length(tempRtV) %use projection on last vintage to fill up sample
            
            tempY =tempV(iV & iLV); 
            tempX =tempLV(iV & iLV);
            fillT =length(lastVrefP(lastVrefP>=startE & lastVrefP<firstDataPointInV));
            
            tempV =realTimeOriginalSet(realTimeSetRefP>=Tstart,t+startV-1);
            
            endDataVraw(1:fillT+length(tempV),NoV-length(tempRtV)+t)=...
                [ [ones(fillT,1) lastV(1:fillT)]*([ones(size(tempX)) tempX]\tempY); tempV];

            
        else 
            
            tempNextV    =realTimeOriginalSet(realTimeSetRefP>=Tstart & realTimeSetRefP<=Tend,t+1+startV-1);

            isNotRevised =tempV-tempNextV; 
            isNotRevised(isnan(isNotRevised))=0; 
            isNotRevised =all(isNotRevised==0);

            if isNotRevised %cut previously updated vintage
                
                idx =find(~isnan(endDataVraw(:,NoV-length(tempRtV)+t+1)),1,'last');
                
                endDataVraw(1:idx-1,NoV-length(tempRtV)+t)=endDataVraw(1:idx-1,NoV-length(tempRtV)+t+1);
                
            else
            
                tempY =tempV(iV & iLV); 
                tempX =tempLV(iV & iLV);
                fillT =length(lastVrefP(lastVrefP>=startE & lastVrefP<firstDataPointInV));
                
                tempV =realTimeOriginalSet(realTimeSetRefP>=Tstart,t+startV-1);
                
                endDataVraw(1:fillT+length(tempV),NoV-length(tempRtV)+t)=...
                    [ [ones(fillT,1) lastV(1:fillT)]*([ones(size(tempX)) tempX]\tempY); tempV];
            end
        end
    end
    
    firstDataPointInV =firstDataPointInLastV; 
    
    case2=true;
end



%case3:  the series starts at a later date than the estimation date but
%======  still before the beginning of the OOS span --> fill up initial space 
%        with missing values

if   ~case2 && min(firstDataPointInLastV,firstDataPointInV)>startE 
    
    if strcmp(baseInfo.dataFrequency(iS),'M')

        %record action in build log
        fprintf(fid,'(!) first available data point is %s \r\n',datestr(min(firstDataPointInLastV,firstDataPointInV)));
        fprintf(fid,'(!) using NaNs to fill up estimation span from %s\r\n\r\n',datestr(startE));

        idx =find(ismember(lastVrefP,firstDataPointInV));

        endDataVraw(idx:idx-1+size(realTimeOriginalSet,1),NoPV+1:end)=...
            realTimeOriginalSet(:,ismember(realTimeSetDates,endDatesV));
        
        firstDataPointInV=lastVrefP(1);
        
    elseif strcmp(baseInfo.dataFrequency(iS),'Q')
       
        %record action in build log
        fprintf(fid,'(!) first available data point is %s \r\n',datestr(min(firstDataPointInLastV,firstDataPointInV)));
        fprintf(fid,'(!) using NaNs to fill up estimation span from %s\r\n\r\n',datestr(startE));

        idx =find(ismember(lastVrefP,firstDataPointInV));

        endDataVraw(idx:idx-1+size(realTimeOriginalSet,1),NoPV+1:end)=...
            realTimeOriginalSet(:,ismember(realTimeSetDates,endDatesV));
        
        firstDataPointInV=lastVrefP(1);
        
    end    
end



%case4:  the starting date of real time vintages is changing (increasing)
%======  over time --> substitute initial missing as in case2

if isnan(any(endDataVraw(1,:))) && not(all(isnan(endDataVraw(1,:)))) && strcmp(baseInfo.dataFrequency(iS),'M')
    
    raggedStart =find(isnan(endDataVraw(1,:)),1,'first');

    %record action in build log
    fprintf(fid,'(!) data in real time vintages start at different dates\r\n');
    fprintf(fid,'(!) filling up from %s using projection on last vintage \r\n\r\n',datestr(endDatesV(raggedStart)));

    for t=NoV:-1:raggedStart
        
        tempV  =endDataVraw(:,t); 
        iV     =~isnan(tempV); 
        temp   =find(~isnan(tempV),1,'first');

        Tstart =max(firstDataPointInLastV,realTimeSetRefP(temp)); 
        Tend   =min(lastVrefP(end),realTimeSetRefP(end));
    
        tempLV =lastV(lastVrefP>=Tstart & lastVrefP<=Tend); 
        iLV    =~isnan(tempLV);

        tempY  =tempV(iV & iLV); 
        tempX  =tempLV(iV & iLV);
        fillT  =length(lastVrefP(lastVrefP>=startE & lastVrefP<firstDataPointInV));

        endDataVraw(1:fillT+length(tempV),NoV-length(tempRtV)+t)=...
        [ [ones(fillT,1) lastV(1:fillT)]*([ones(size(tempX)) tempX]\tempY); tempV];
        
    end
    
end



%case5:  the real time vintages do not cover the entire OOS span --> append
%======  real vintages to pseudo ones constructed backward from first available

if firstVdate>startF && firstDataPointInLastV<firstVdate 
    
    if strcmp(baseInfo.dataFrequency(iS),'M')

        %record action in build log
        fprintf(fid,'(!) first real time vintage released on %s \r\n',datestr(realTimeSetDates(1)));
        fprintf(fid,'(!) building pseudo vintages to fill up OOS span from %s\r\n\r\n',datestr(startF));

        staticV =endDataVraw(:,find(all(isnan(endDataVraw)),1,'last')+1);
        staticV =staticV(1:find(~isnan(staticV),1,'last')); fillT=length(staticV);

        idx =find(ismember(lastVrefP,firstDataPointInV));
        for t=NoPV:-1:1

            endDataVraw(idx:fillT+t-NoPV-2+idx,t)=staticV(1:fillT+t-NoPV-1);

        end
    
    elseif  strcmp(baseInfo.dataFrequency(iS),'Q')
        
        %record action in build log
        fprintf(fid,'(!) first real time vintage released on %s \r\n',datestr(realTimeSetDates(1)));
        fprintf(fid,'(!) building pseudo vintages to fill up OOS span from %s\r\n\r\n',datestr(startF));
        
        staticV =endDataVraw(:,find(all(isnan(endDataVraw)),1,'last')+1);
        staticV =staticV(1:find(~isnan(staticV),1,'last')); fillT=length(staticV);

        idx =find(ismember(lastVrefP,firstDataPointInV));
        for t=NoPV:-1:1

            endDataVraw(idx:fillT+t-NoPV-2+idx,t)=staticV(1:fillT+t-NoPV-1);

        end
        
    end
end





%apply transformations
endDataV        =endDataVraw; 
labelTransform  ='none';

if strcmp(baseInfo.dataFrequency(iS),'M')
    
    if iTransform(1) && iTransform(2)
        endDataV(2:end,:)=100*diff(log(endDataVraw)); 
        endDataV(1,:)    =nan;
        labelTransform   ='logdiff';
        
    elseif not(iTransform(1)) && iTransform(2)
        endDataV(2:end,:)=diff(endDataVraw); 
        endDataV(1,:)    =nan;
        labelTransform   ='diff';
    end

    if iTransform(3)
        endDataV         =filter(ones(12,1),1,endDataV);
        labelTransform   ='yoyfilter';
    end
    
elseif strcmp(baseInfo.dataFrequency(iS),'Q')
    
    if iTransform(1) && iTransform(2)
        endDataV(6:3:end,:) =100*diff(log(endDataVraw(3:3:end,:))); 
        endDataV(3,:)       =nan;
        labelTransform      ='logdiff';
        
    elseif not(iTransform(1)) && iTransform(2)
        endDataV(6:3:end,:) =diff(log(endDataVraw(3:3:end,:))); 
        endDataV(3,:)       =nan;
        labelTransform      ='diff';
    end

    if iTransform(3)
        endDataV(6:3:end,:) =filter(ones(4,1),1,endDataV(6:3:end));
        labelTransform      ='yoyfilter';
    end    
end


%reference data points
endPeriods =nan(NoP,1); 
nP         =length(lastVrefP);

if nP<NoP
    
    endPeriods(1:nP)=lastVrefP;
    for j=nP+1:NoP
        
        endPeriods(j)=addtodate(endPeriods(j-1),1,'month');
    end
else
    endPeriods=lastVrefP(1:NoP);
end


%plot revisions on transformed variables
if baseInfo.plotVariables
    
    figure;
    subplot(2,1,1)
    plotSet =endDataV(endPeriods<=endF,NoPV+1:end); 
    if strcmp(baseInfo.dataFrequency(iS),'Q')
        plotSet =kron(plotSet(3:3:end,:),ones(3,1));
    end
    
    plotDates=endPeriods(endPeriods<=endF);
    plot(plotDates,plotSet)
    set(gca,'XTick',plotDates(1:60:end),'FontSize',8,'Layer','top'); datetick('x','yyyy','keepticks');
    grid on; xlim([plotDates(1) plotDates(end)]); ylim([-inf inf]);
    title([idCode,' :: model transform = ',labelTransform],'FontSize',9,'FontWeight','normal')
    ll=legend('real-time vintages'); set(ll,'FontSize',9,'Location','SouthWest');
    ll=findobj(ll,'type','line','linestyle','-'); set(ll,'visible','off');
    %
    subplot(2,1,2)
    plotSet=diff(endDataV(endPeriods<=endF,NoPV+1:end)')'; 
    bar(plotDates,plotSet,.5,'stack'); 
    cMapStart=[0 .4 .2]'; cMapMid=[.4 .2 .8]'; cMapEnd=[1 .4 0]'; nV=size(plotSet,2);
    cMap=[linspace(cMapStart(1),cMapMid(1),nV/2),linspace(cMapMid(1),cMapEnd(1),nV/2);...
        linspace(cMapStart(2),cMapMid(2),nV/2),linspace(cMapMid(2),cMapEnd(2),nV/2);
        linspace(cMapStart(3),cMapMid(3),nV/2),linspace(cMapMid(3),cMapEnd(3),nV/2);]';
    colormap(cMap); %colormap winter;
    cb=colorbar('Location','South','FontSize',8); 
    cbP=get(cb,'Position'); cbP(3)=.4; cbP(4)=.03; set(cb,'Position',cbP)
    tempLabels=cellstr(datestr(floor(endDatesV(NoPV+1:end)),'yyyy'))';
    set(cb,'XTickLabel',tempLabels(1:20:end))
    set(gca,'XTick',plotDates(1:60:end),'FontSize',8,'Layer','top'); datetick('x','yyyy','keepticks');
    grid on; xlim([plotDates(1) plotDates(end)]); ylim([-inf inf]);    
    title([idCode,' :: revision history'],'FontSize',9,'FontWeight','normal')    
    
    set(gcf,'PaperPosition',[-1.5 -1 21 15]) %leftMargin topMargin figureLength figureHeight
    set(gcf,'PaperSize', [18 14]); %xSize ySize
    print(gcf,'-dpdf',[baseInfo.chartsFolderName,'/',idCode,'rev.pdf']);
    
    close(gcf)

end


%load results structure
res.dataVintages     =endDataV;     %data vintages
res.releaseDates     =endDatesV;    %release dates
res.referencePeriods =endPeriods;   %reference periods





%--------------------------------------------------------------------------

function res=buildPseudoRTimeVintage(baseInfo,idCode,lastVintage,releaseCal)


%unpack general inputs
fid    =baseInfo.logFileID;
startE =baseInfo.startEstimate;
startF =baseInfo.startForecast; endF=baseInfo.endForecast;

%unpack series-specific information
iS             =ismember(baseInfo.dataList,idCode);
iTransform     =baseInfo.dataTransform(iS,:); refP=releaseCal.refP; iCal=releaseCal.releaseDates(:,iS);
res.dataSelect =baseInfo.dataSelect; 


%last vintage
lastV     =lastVintage.data(:,iS); 
lastVrefP =lastVintage.dates; 


%trim unnecessary observations (last vintage)
if startE>lastVrefP(1)
    
    cutData   =lastVrefP>=startE; 
    lastV     =lastV(cutData); 
    lastVrefP =lastVrefP(cutData);
end


%remove series if first data point released after the beginning of the
%evaluation sample
firstDataPointInLastV =lastVrefP(find(~isnan(lastV),1,'first'));

if firstDataPointInLastV>startF
    
    %record action in build log
    fprintf(fid,'(!) not enough data points for %s: removing series from set (!)\r\n',idCode);
    temp =find(lastVrefP==firstDataPointInLastV);
    fprintf(fid,'(!) push start of OOS to at least %s to include it (!)\r\n',datestr(lastVrefP(temp+6)));
    
    %remove from set
    res.dataSelect(iS)=false;  
    return
end

%time bounds
NoP       =max(length(lastVrefP),ceil(1.1*(endF-startE)/365*12)); %number of periods
endDatesV =iCal(iCal>=startF & iCal<=endF)';
NoV       =length(endDatesV);                                     %number of (pseudo) vintages

matchRelRefP =refP(find(ismember(iCal,endDatesV),1,'last'));

endLV     =lastVrefP<=matchRelRefP; 
lastVrefP =lastVrefP(endLV); 
lastV     =lastV(endLV); 
nT        =find(~isnan(lastV),1,'last');

%build final data vintage set using pseudo releases
endDataVraw =nan(NoP,NoV); endDataVraw(1:nT,end)=lastV;

if strcmp(baseInfo.dataFrequency(iS),'Q')
    for t=NoV:-1:2

        endDataVraw(1:nT+(t-NoV)*3-3,t-1)=lastV(1:nT+(t-NoV)*3-3);
    end
else
    for t=NoV:-1:2

        endDataVraw(1:nT+t-NoV-1,t-1)=lastV(1:nT+t-NoV-1);
    end
end

%apply transformations
endDataV =endDataVraw;

if strcmp(baseInfo.dataFrequency(iS),'M')
    
    if iTransform(1) && iTransform(2)
        endDataV(2:end,:)=100*diff(log(endDataVraw)); 
        endDataV(1,:)    =nan;
        
    elseif not(iTransform(1)) && iTransform(2)
        endDataV(2:end,:)=diff(endDataVraw); 
        endDataV(1,:)    =nan;
    end

    if iTransform(3)
        endDataV         =filter(ones(12,1),1,endDataV);
    end
    
elseif strcmp(baseInfo.dataFrequency(iS),'Q')
    
    if iTransform(1) && iTransform(2)
        endDataV(6:3:end,:) =100*diff(log(endDataVraw(3:3:end,:))); 
        endDataV(3,:)       =nan;
        
    elseif not(iTransform(1)) && iTransform(2)
        endDataV(6:3:end,:) =diff(log(endDataVraw(6:3:end,:))); 
        endDataV(3,:)       =nan;
    end

    if iTransform(3)
        endDataV(6:3:end,:) =filter(ones(4,1),1,endDataV(6:3:end));
    end    
end



%reference data points
endPeriods =nan(NoP,1); 
nP         =length(lastVrefP);
if nP<NoP
    
    endPeriods(1:nP)=lastVrefP;
    for j=nP+1:NoP
        
        endPeriods(j)=addtodate(endPeriods(j-1),1,'month');
    end
else
    endPeriods=lastVrefP(1:NoP);
end

endDataV(endDataV==inf) = 0;
endDataV(endDataV==-inf) = 0;

%load results structure
res.dataVintages     =endDataV;     %data vintages
res.releaseDates     =endDatesV;    %release dates
res.referencePeriods =endPeriods;   %reference periods
