
function dataStructure=readStaticData(dataInfo)

% reads from xl file which contains a combination of:
% (1) last vintage data for monthly variables 
% (2) real time diagonal and main off-diagonal for GDP
% returns an ordered set of vintages and their timestamp
%
% miranda (2016) silvia.miranda-agrippino@bankofengland.co.uk



startT      =dataInfo.startEstimate;
rawDataFile =dataInfo.rawDataFile;

% . read summary tab  . . . . . . . . . . . . . . . . . . . . . . . . . . %

[tempData,tempText]       =xlsread(rawDataFile,'summary');

dataName        =tempText(2:end,1);
dataDescription =tempText(2:end,2);
dataSource      =tempText(2:end,3);
dataFrequency   =tempText(2:end,4);
dataGroups      =tempText(2:end,5);

isRealTime      =logical(tempData(:,1));
isGDPrelease    =logical(tempData(:,2));
dataSelect      =logical(tempData(:,3));
dataTransform   =logical(tempData(:,4:6));

dataBlocks      =logical(tempData(:,7:end));
dataBlocks(:,all(~dataBlocks))=[];

% . read release calendar . . . . . . . . . . . . . . . . . . . . . . . . %

[tempData,~]               =xlsread(rawDataFile,'releaseCalendar');

releaseCalendar.dates      =x2mdate(tempData(:,2:end)); %release dates
releaseCalendar.referenceP =x2mdate(tempData(:,1));     %reference period



% . read data . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . %

[tempData,~]               =xlsread(rawDataFile,'staticVintage');

staticVintage.data           =tempData(:,2:end);          %mixture of real time and last vintages
staticVintage.dates          =x2mdate(tempData(:,1));


%. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .%

%apply transformations 

isM   =ismember(dataFrequency,'M');       nM=sum(isM);
isQ   =ismember(dataFrequency,'Q');       nQ=sum(isQ);

Mdata =staticVintage.data(:,isM);
Qdata =staticVintage.data(:,isQ);

%monthly
Mdata(:,dataTransform(isM,1)) =100*log(Mdata(:,dataTransform(isM,1)));

Mdata(:,dataTransform(isM,2)) =[nan(1,sum(dataTransform(isM,2))); ...
    diff(Mdata(:,dataTransform(isM,2)))];


Mdata(:,dataTransform(isM,3)) =filter(ones(12,1),1,Mdata(:,dataTransform(isM,3)));


%quarterly
isEOQ =mod(month(staticVintage.dates),3)==0; %last month in quarter

Qdata =Qdata(isEOQ,:);
Qdates=staticVintage.dates(isEOQ);

Qdata(:,dataTransform(isQ,1)) =100*log(Qdata(:,dataTransform(isQ,1)));

Qdata(:,dataTransform(isQ,2)) =[nan(1,sum(dataTransform(isQ,2))); ...
    diff(Qdata(:,dataTransform(isQ,2)))];


%assemble dataset
T=numel(staticVintage.dates);

dataMatrix =nan(T, nM+nQ);

dataMatrix(:,1:nM)=Mdata;
dataMatrix(ismember(staticVintage.dates,Qdates),nM+1:end)=Qdata;


%select relevant sample
selectT =staticVintage.dates>=startT;
selectN =dataSelect;

dates   =staticVintage.dates(staticVintage.dates>=startT);



if dataInfo.plotVariables
    for j=1:size(dataMatrix,2)
        
        if strcmp(dataFrequency(j),'M')
            
            plot(staticVintage.dates,dataMatrix(:,j),'LineWidth',1.5); 
        else
            
            tempY =nan(size(staticVintage.dates));
            tempY(1:3*floor(numel(dataMatrix(:,j))/3)) =kron(dataMatrix(3:3:end,j),ones(3,1));
            plot(staticVintage.dates,tempY,'LineWidth',1.5); 
        end
        
        grid on; axis tight; dateaxis('x',10)
        title(dataDescription{j},'FontSize',11,'FontWeight','normal')
%         pause
        
        set(gcf,'PaperPosition',[-.5 0 14 8]) %leftMargin topMargin figureLength figureHeight
        set(gcf,'PaperSize', [13 8]); %xSize ySize
        print(gcf,'-dpdf',[pwd '/DATA/' dataName{j} '.pdf']);
        
        close gcf
    end
end



%. load data structure . . . . . . . . . . . . . . . . . . . . . . . . . .%

dataStructure.dataMatrix      =dataMatrix;
dataStructure.dataInfo        =dataInfo;
dataStructure.dates           =dates;

%data info
dataStructure.isRealTime      =isRealTime;
dataStructure.isGDPrelease    =isGDPrelease;
dataStructure.selectT         =selectT;
dataStructure.selectN         =selectN;
dataStructure.dataName        =dataName;
dataStructure.dataDescription =dataDescription;
dataStructure.dataSource      =dataSource;
dataStructure.dataFrequency   =dataFrequency;
dataStructure.dataTransform   =dataTransform;
dataStructure.dataGroups      =dataGroups;

dataStructure.blocks          =dataBlocks;

%calendar
dataStructure.calendar.dates  =releaseCalendar.dates;
dataStructure.calendar.refP   =releaseCalendar.referenceP;

%counters
dataStructure.nT =T;
dataStructure.nM =sum(ismember(dataFrequency,'M'));
dataStructure.nQ =sum(ismember(dataFrequency,'Q'));


save([pwd '/DATA/' dataInfo.dataFile],'dataStructure');




